@@ -179,8 +179,6 @@ function Audio:init()
 end
 
 function Audio:initSpeech(speech_file)
-  self.sound_archive = nil
-  self.sound_fx = nil
   if self.not_loaded then
     return
   end
@@ -206,6 +204,10 @@ function Audio:initSpeech(speech_file)
     archive_data = load_sound_file(speech_file)
   end
 
+  self.sound_archive = nil
+  self.sound_fx = nil
+  self.speech_file_name = nil
+
   if not archive_data then
     if self.app.good_install_folder then
       print("Notice: No sound effects as no SOUND/DATA/".. speech_file ..
@@ -226,13 +228,20 @@ function Audio:initSpeech(speech_file)
       self.speech_file_name = speech_file
       self.sound_fx = TH.soundEffects()
       self.sound_fx:setSoundArchive(self.sound_archive)
-      local w, h = self.app.config.width / 2, self.app.config.height / 2
-      self.sound_fx:setCamera(math.floor(w), math.floor(h), math.floor((w^2 + h^2)^0.5))
+      self:setSoundStage()
       --self:dumpSoundArchive[[E:\CPP\2K8\CorsixTH\DataRaw\Sound\]]
     end
   end
 end
 
+--! Set the visual area for sound effects playback
+function Audio:setSoundStage()
+  if self.sound_fx then
+    local w, h = self.app.config.width / 2, self.app.config.height / 2
+    self.sound_fx:setCamera(math.floor(w), math.floor(h), math.floor((w^2 + h^2)^0.5))
+  end
+end
+
 function Audio:dumpSoundArchive(out_dir)
   local info,warning = io.open(out_dir .. "info.csv", "w")
 
