@@ -142,7 +142,9 @@ function UIBottomPanel:registerKeyHandlers()
 end
 
 function UIBottomPanel:openJukebox()
-  self.ui:addWindow(UIJukebox(self.ui.app))
+  if self.ui.app.config.audio and self.ui.app:isAudioEnabled() then
+    self.ui:addWindow(UIJukebox(self.ui.app))
+  end
 end
 
 function UIBottomPanel:openSave()
@@ -201,7 +203,7 @@ function UIBottomPanel:draw(canvas, x, y)
   self:drawReputationMeter(canvas, x + 55, y + 35)
 end
 
-function UIBottomPanel:setPosition(x, y)
+function UIBottomPanel:setPosition(x, y) -- luacheck: ignore 212 keep y from parent class
   -- Lock to bottom of screen
   return Window.setPosition(self, x, -0.1)
 end
@@ -461,7 +463,7 @@ end
 --! Pop the message with the given index from the message queue and turn it into an actual
 -- message window; if no index is provided the first message in the queue is popped.
 function UIBottomPanel:createMessageWindow(index)
-  local --[[persistable:bottom_panel_message_window_close]] function onClose(window, out_of_time)
+  local --[[persistable:bottom_panel_message_window_close]] function onClose(window)
     local index_to_remove
     for i, win in ipairs(self.message_windows) do
       if index_to_remove ~= nil then
@@ -484,6 +486,7 @@ function UIBottomPanel:createMessageWindow(index)
   if not message_info then
     return
   end
+  -- Create the message window, note this does not show it to the player on creation.
   local alert_window = UIMessage(self.ui, 175, 1 + #message_windows * 30,
     onClose, message_info.type, message_info.message, message_info.owner, message_info.timeout, message_info.default_choice, message_info.callback)
   message_windows[#message_windows + 1] = alert_window
@@ -698,6 +701,9 @@ function UIBottomPanel:dialogResearch(enable)
     self:updateButtonStates()
     return
   end
+  if TheApp.using_demo_files then
+    self.ui:addWindow(UIInformation(self.ui, {_S.errors.dialog_missing_graphics}))
+  end
   if self.ui.hospital.research_dep_built then
     if enable then
       self:addDialog("UIResearch")
@@ -724,7 +730,9 @@ function UIBottomPanel:giveResearchAdvice()
     end
   end
   local msg = can_build_research and _A.warnings.research_screen_open_1 or _A.warnings.research_screen_open_2
-  self.ui.adviser:say(msg)
+  if not TheApp.using_demo_files then
+    self.ui.adviser:say(msg)
+  end
 end
 
 function UIBottomPanel:dialogStatus(enable)
@@ -797,7 +805,7 @@ function UIBottomPanel:addDialog(dialog_class, extra_function)
   local edit_window = self.ui:getWindow(UIEditRoom)
   -- If we are currently editing a room, ask for abortion before adding any dialog.
   if edit_window then
-    self.ui:addWindow(UIConfirmDialog(self.ui,
+    self.ui:addWindow(UIConfirmDialog(self.ui, false,
       _S.confirmation.abort_edit_room,
       --[[persistable:abort_edit_room_confirm_dialog]]function()
         self.ui:setEditRoom(false)
@@ -863,9 +871,10 @@ function UIBottomPanel:afterLoad(old, new)
     end
     self.bank_button = self.buttons[1]:makeToggle()
   end
+  -- Hotfix to force re-calculation of the money font (see issue #1193)
+  self.money_font = self.ui.app.gfx:loadFont("QData", "Font05V")
 
   self:registerKeyHandlers()
 
   Window.afterLoad(self, old, new)
 end
-
