@@ -159,7 +159,6 @@ function UIEditRoom:abortRoom()
   self:close()
   -- Finally remove the room from the world (close() needs the reference)
   if self.room then
-    self.room:deactivate() -- TODO: may be superfluous since already called in Room:tryToEdit
     self.world.rooms[self.room.id] = nil
   end
 end
@@ -688,7 +687,7 @@ function UIEditRoom:returnToDoorPhase()
   local room = self.room
   room.built = false
   if room.door and room.door.queue then
-    room.door.queue:rerouteAllPatients(SeekRoomAction(room.room_info.id))
+    room.door.queue:rerouteAllPatients(room.room_info.id)
   end
 
   self.purchase_button:enable(false)
@@ -1447,9 +1446,9 @@ function UIEditRoom:onCursorWorldPositionChange(x, y)
         self.move_rect = false
         self.resize_rect = {
           n = (wy == rect.y),
-          s = (wy == rect.y + rect.h - 1) and not (wy == rect.y),
+          s = (wy == rect.y + rect.h - 1) and (wy ~= rect.y),
           w = (wx == rect.x),
-          e = (wx == rect.x + rect.w - 1) and not (wx == rect.x),
+          e = (wx == rect.x + rect.w - 1) and (wx ~= rect.x),
         }
 
         if (self.resize_rect.w or self.resize_rect.e) and (self.resize_rect.n or self.resize_rect.s) then
