@@ -160,13 +160,14 @@ function UIAnnualReport:UIAnnualReport(ui, world)
     table.sort(self.value_sort, desc_order)
     table.sort(self.salary_sort, desc_order)
 
-  -- Pause the game to allow the player plenty of time to check all statistics and trophies won
-  if world and not world:isCurrentSpeed("Pause") then
-    world:setSpeed("Pause")
-  end
   TheApp.video:setBlueFilterActive(false)
 end
 
+-- Make sure this window pauses the game, we want to let the player browse their awards
+function UIAnnualReport:mustPause()
+  return true
+end
+
 --! Finds out which awards and/or trophies the player has been awarded this year.
 function UIAnnualReport:checkTrophiesAndAwards(world)
 
@@ -200,12 +201,15 @@ function UIAnnualReport:checkTrophiesAndAwards(world)
       self.rep_amount = self.rep_amount + win_value
     end
     -- Impressive Reputation in the year (above a threshold throughout the year)
-    if hosp.reputation_above_threshold then
+    if hosp.has_impressive_reputation then
       self:addTrophy(_S.trophy_room.consistant_rep.trophies[math.random(1, 2)], "money", prices.TrophyReputationBonus)
       self.won_amount = self.won_amount + prices.TrophyReputationBonus
     end
-    -- No deaths or around a 100% Cure rate in the year
-    if hosp.num_deaths_this_year == 0 then
+    -- Everyone treated successfully, no deaths, or around a 100% Cure rate in the year
+    if hosp.num_cured_ty > 1 and hosp.num_deaths_this_year == 0 and hosp.not_cured_ty == 0 then
+      self:addTrophy(_S.trophy_room.all_cured.trophies[math.random(1, 2)], "money", prices.TrophyAllCuredBonus)
+      self.won_amount = self.won_amount + prices.TrophyAllCuredBonus
+    elseif hosp.num_deaths_this_year == 0 then
       self:addTrophy(_S.trophy_room.no_deaths.trophies[math.random(1, 3)], "money", prices.TrophyDeathBonus)
       self.won_amount = self.won_amount + prices.TrophyDeathBonus
     elseif hosp.num_cured_ty > (hosp.not_cured_ty * 0.9)  then
@@ -251,7 +255,10 @@ function UIAnnualReport:checkTrophiesAndAwards(world)
     end
 
     -- Deaths
-    if hosp.num_deaths_this_year < prices.DeathsAward then
+    if hosp.num_cured_ty > 1 and hosp.num_deaths_this_year == 0 and hosp.not_cured_ty == 0 then
+      self:addAward(_S.trophy_room.no_deaths.awards[1], "money", prices.AllCuresBonus)
+      self.award_won_amount = self.award_won_amount + prices.AllCuresBonus
+    elseif hosp.num_deaths_this_year < prices.DeathsAward then
       self:addAward(_S.trophy_room.no_deaths.awards[math.random(1, 2)], "money", prices.DeathsBonus)
       self.award_won_amount = self.award_won_amount + prices.DeathsBonus
     elseif hosp.num_deaths_this_year > prices.DeathsPoor then
@@ -340,7 +347,7 @@ function UIAnnualReport:addTrophy(text, award_type, amount)
     trophy_parts.info = {
       text = text,
       award_type = award_type,
-      amount = amount
+      amount = math.floor(amount)
     }
 
     local --[[persistable:annual_report_show_trophy_motivation]] function change() self:showTrophyMotivation(no) end
@@ -376,7 +383,7 @@ function UIAnnualReport:addAward(text, award_type, amount)
     award_parts.info = {
       text = text,
       award_type = award_type,
-      amount = amount
+      amount = math.floor(amount)
     }
 
     -- The plaque
@@ -445,17 +452,8 @@ end
 --! Overridden close function. The game should be unpaused again when closing the dialog.
 function UIAnnualReport:close()
   if TheApp.world:getLocalPlayerHospital().game_won then
-    if not TheApp.world:isCurrentSpeed("Pause") then
-      TheApp.world:setSpeed("Pause")
-      TheApp.video:setBlueFilterActive(false)
-    end
+    TheApp.video:setBlueFilterActive(false)
     TheApp.world.ui.bottom_panel:openLastMessage()
-  elseif TheApp.world:isCurrentSpeed("Pause") then
-    if TheApp.ui.speed_up_key_pressed then
-      TheApp.world:setSpeed("Speed Up")
-    else
-      TheApp.world:setSpeed(TheApp.world.prev_speed)
-    end
   end
   self:updateAwards()
   Window.close(self)
@@ -523,6 +521,7 @@ function UIAnnualReport:draw(canvas, x, y)
   elseif self.state == 2 then -- Statistics screen
     self:drawStatisticsScreen(canvas, x, y)
   else -- Award and trophy screen
+    -- Note: Each 'amount' is wrapped by string.format("%.0f", x) to persist integer notation
     -- Write out motivation if appropriate
     if self.trophy_motivation then
       -- If it is a plaque showing we write in stone text.
@@ -535,7 +534,7 @@ function UIAnnualReport:draw(canvas, x, y)
       end
       self.stone_font:draw(canvas, award_type, x + 220, y + 330, 200, 0)
       -- Amount won/lost
-      self.stone_font:draw(canvas, "+" .. info.amount, x + 220, y + 355, 200, 0)
+      self.stone_font:draw(canvas, string.format("+%.0f", info.amount), x + 220, y + 355, 200, 0)
     elseif self.award_motivation then
       local info = self.awards[self.award_motivation].info
       self.write_font:drawWrapped(canvas, info.text, x + 235, y + 125, 165, "center")
@@ -546,11 +545,7 @@ function UIAnnualReport:draw(canvas, x, y)
       end
       self.write_font:draw(canvas, award_type, x + 220, y + 290, 200, 0)
       -- The amount won/lost
-      local text = ""
-      if info.amount > 0 then
-        text = "+"
-      end
-      self.write_font:draw(canvas, text .. info.amount, x + 220, y + 315, 200, 0)
+      self.write_font:draw(canvas, string.format("%+.0f", info.amount), x + 220, y + 315, 200, 0)
     end
   end
 end
@@ -590,6 +585,7 @@ function UIAnnualReport:drawStatisticsScreen(canvas, x, y)
   for i, hospital in ipairs(world.hospitals) do
     local name = hospital.name
 
+    -- Note: Each 'value' is wrapped by string.format("%.0f", x) to persist integer notation
     -- Most Money
     local index_m = getindex(self.money_sort, i)
     -- index_* is the returned value of the sorted place for this player.
@@ -597,42 +593,42 @@ function UIAnnualReport:drawStatisticsScreen(canvas, x, y)
     -- duplicate has been found, one additional row lower is the right place to be.
     font:draw(canvas, name:upper(), x + 140,
         y + row_y + row_dy * (index_m - 1))
-    font:draw(canvas, self.money_sort[index_m].value, x + 240,
+    font:draw(canvas, string.format("%.0f", self.money_sort[index_m].value), x + 240,
         y + row_y + row_dy * (index_m - 1), 70, 0, "right")
 
     -- Highest Salary
     local index_s = getindex(self.salary_sort, i)
     font:draw(canvas, name:upper(), x + 140 + col_x,
         y + row_y + row_dy * (index_s - 1))
-    font:draw(canvas, self.salary_sort[index_s].value, x + 240 + col_x,
+    font:draw(canvas, string.format("%.0f", self.salary_sort[index_s].value), x + 240 + col_x,
         y + row_y + row_dy * (index_s - 1), 70, 0, "right")
 
     -- Most Cures
     local index_c = getindex(self.cures_sort, i)
     font:draw(canvas, name:upper(), x + 140,
         y + row_y + row_no_y + row_dy * (index_c - 1))
-    font:draw(canvas, self.cures_sort[index_c].value, x + 240,
+    font:draw(canvas, string.format("%.0f", self.cures_sort[index_c].value), x + 240,
         y + row_y + row_no_y + row_dy * (index_c - 1), 70, 0, "right")
 
     -- Most Deaths
     local index_d = getindex(self.deaths_sort, i)
     font:draw(canvas, name:upper(), x + 140 + col_x,
         y + row_y + row_no_y + row_dy * (index_d - 1))
-    font:draw(canvas, self.deaths_sort[index_d].value, x + 240 + col_x,
+    font:draw(canvas, string.format("%.0f", self.deaths_sort[index_d].value), x + 240 + col_x,
         y + row_y + row_no_y + row_dy * (index_d - 1), 70, 0, "right")
 
     -- Most Visitors
     local index_v = getindex(self.visitors_sort, i)
     font:draw(canvas, name:upper(), x + 140,
         y + row_y + row_no_y * 2 + row_dy * (index_v - 1))
-    font:draw(canvas, self.visitors_sort[index_v].value, x + 240,
+    font:draw(canvas, string.format("%.0f", self.visitors_sort[index_v].value), x + 240,
         y + row_y + row_no_y * 2 + row_dy * (index_v - 1), 70, 0, "right")
 
     -- Highest Value
     local index_v2 = getindex(self.value_sort, i)
     font:draw(canvas, name:upper(), x + 140 + col_x,
         y + row_y + row_no_y * 2 + row_dy * (index_v2 - 1))
-    font:draw(canvas, self.value_sort[index_v2].value, x + 240 + col_x,
+    font:draw(canvas, string.format("%.0f", self.value_sort[index_v2].value), x + 240 + col_x,
         y + row_y + row_no_y * 2 + row_dy * (index_v2 - 1), 70, 0, "right")
   end
 end
