@@ -149,7 +149,6 @@ end
 function UICasebook:updateIcons()
   local disease = self.selected_disease
   local hosp = self.hospital
-  local world = hosp.world
 
   local known = true
   -- Curable / not curable icons and their tooltip
@@ -170,31 +169,21 @@ function UICasebook:updateIcons()
       local build = false
       local staff = false
       -- Room requirements
-      if #req.rooms > 0 then
-        for _, room_id in ipairs(req.rooms) do
-          -- Not researched yet?
-          if not hosp.discovered_rooms[world.available_rooms[room_id]] then
-            known = false
-            research = (research and (research .. ", ") or " (") .. TheApp.rooms[room_id].name
-          end
-          -- Researched, but not built. TODO: maybe make this an else clause to not oversize the tooltip that much
-          build = (build and (build .. ", ") or " (") .. TheApp.rooms[room_id].name
+      for _, room_id in ipairs(req.rooms) do
+        -- Not researched yet?
+        if not hosp:isRoomDiscovered(room_id) then
+          known = false
+          research = (research and (research .. ", ") or " (") .. TheApp.rooms[room_id].name
         end
+        -- Researched, but not built. TODO: maybe make this an else clause to not oversize the tooltip that much
+        build = (build and (build .. ", ") or " (") .. TheApp.rooms[room_id].name
       end
       research = research and (_S.tooltip.casebook.cure_requirement.research_machine .. research .. "). ") or ""
       build    = build    and (_S.tooltip.casebook.cure_requirement.build_room .. build .. "). ") or ""
 
-      local staffclass_to_string = {
-        Nurse        = _S.staff_title.nurse,
-        Doctor       = _S.staff_title.doctor,
-        Surgeon      = _S.staff_title.surgeon,
-        Psychiatrist = _S.staff_title.psychiatrist,
-        Researcher   = _S.staff_title.researcher,
-      }
-
       -- Staff requirements
       for sclass, amount in pairs(req.staff) do
-        staff = (staff and (staff .. ", ") or " (") .. staffclass_to_string[sclass] .. ": " .. amount
+        staff = (staff and (staff .. ", ") or " (") .. StaffProfile.translateStaffClass(sclass) .. ": " .. amount
       end
       staff = staff and (_S.tooltip.casebook.cure_requirement.hire_staff .. staff .. "). ") or ""
 
