@@ -55,7 +55,7 @@ function UIStaffManagement:UIStaffManagement(ui)
   self:addPanel(0, 603, 443):makeButton(0, 0, 26, 26, 10, self.close):setTooltip(_S.tooltip.staff_list.close)
 
   -- Top categories
-  local --[[persistable:staff_management_category]] function category(name, state, btn)
+  local --[[persistable:staff_management_category]] function category(name)
     self:setCategory(name)
   end
   self.categories = {
@@ -174,9 +174,8 @@ function UIStaffManagement:updateStaffList(staff_member_removed)
     Handyman = {},
     Receptionist = {},
   }
-  staff_members.Surgeon = staff_members.Doctor
   for _, staff in ipairs(hosp.staff) do
-    local list = staff_members[staff.humanoid_class]
+    local list = staff_members[staff.profile.humanoid_class]
     list[#list + 1] = staff
     -- The selected staff might have been moved because someone else was removed from the list.
     if selected_staff == staff then
@@ -225,7 +224,7 @@ end
 -- Function to select a given staff member.
 -- Includes switching to correct category and page.
 function UIStaffManagement:selectStaff(staff)
-  self:setCategory(staff.humanoid_class == "Surgeon" and "Doctor" or staff.humanoid_class)
+  self:setCategory(staff.profile.humanoid_class)
   for i, s in ipairs(self.staff_members[self.category]) do
     if s == staff then
       self:selectIndex(i)
@@ -278,7 +277,8 @@ function UIStaffManagement:draw(canvas, x, y)
       local row_no = i - (self.page-1)*10
       self.row_blankers[row_no].visible = false
       titles:draw(canvas, row_no + 10*(self.page-1), x + 58, y + 63 + row_no*27)
-      titles:draw(canvas, staff.profile.name,        x + 88, y + 63 + row_no*27)
+      titles:draw(canvas, staff.profile:getFullName(),
+          x + 88, y + 63 + row_no*27)
       titles:draw(canvas, "$" .. staff.profile.wage, x + 230, y + 63 + row_no*27, 80, 0)
 
       -- Draw the morale, tiredness and skill for this staff member
@@ -549,7 +549,7 @@ end
 
 function UIStaffManagement:fire()
   if self.selected_staff then
-    self.ui:addWindow(UIConfirmDialog(self.ui, _S.confirmation.sack_staff, --[[persistable:staff_management_confirm_sack]] function()
+    self.ui:addWindow(UIConfirmDialog(self.ui, false, _S.confirmation.sack_staff, --[[persistable:staff_management_confirm_sack]] function()
       local current_category = self.staff_members[self.category]
       current_category[self.selected_staff]:fire()
       -- Close the staff window if open
