@@ -353,7 +353,7 @@ end
 -- Function which is called when the user clicks on the `Humanoid`.
 --!param ui (GameUI) The UI which the user in question is using.
 --!param button (string) One of: "left", "middle", "right".
-function Humanoid:onClick(ui, button)
+function Humanoid:onClick(ui, button) -- luacheck: ignore 212 keep args from parent class
   if TheApp.config.debug then
     self:dump()
   end
@@ -460,69 +460,18 @@ function Humanoid:getCurrentMood()
   end
 end
 
-local function Humanoid_startAction(self)
+--! Start the next (always first) action in the queue.
+function Humanoid:startAction()
   local action = self.action_queue[1]
 
   -- Handle an empty action queue in some way instead of crashing.
   if not action then
-    -- if this is a patient that is going home, an empty
-    -- action queue is not a problem
-    if class.is(self, Patient) and self.going_home then
-      return
-    end
+    self:_handleEmptyActionQueue() -- Inserts an action into the action queue.
 
-    ---- Empty action queue! ----
-    -- First find out if this humanoid is in a room.
-    local room = self:getRoom()
-    if room then
-      room:makeHumanoidLeave(self)
-    end
-    -- Is it a member of staff, grim or a patient?
-    if class.is(self, Staff) then
-      self:queueAction(MeanderAction())
-    elseif class.is(self,GrimReaper) then
-      self:queueAction(IdleAction())
-    else
-      self:queueAction(SeekReceptionAction())
-    end
-    -- Open the dialog of the humanoid.
-    local ui = self.world.ui
-    if class.is(self, Patient) then
-      ui:addWindow(UIPatient(ui, self))
-    elseif class.is(self, Staff) then
-      ui:addWindow(UIStaff(ui, self))
-    end
-
-    -- Tell the player what just happened.
-    self.world:gameLog("")
-    self.world:gameLog("Empty action queue!")
-    self.world:gameLog("Last action: " .. self.previous_action.name)
-    self.world:gameLog(debug.traceback())
-
-    ui:addWindow(UIConfirmDialog(ui, true,
-      "Sorry, a humanoid just had an empty action queue,"..
-      " which means that he or she didn't know what to do next."..
-      " Please consult the command window for more detailed information. "..
-      "A dialog with "..
-      "the offending humanoid has been opened. "..
-      "Would you like him/her to leave the hospital?",
-      --[[persistable:humanoid_leave_hospital]] function()
-        self.world:gameLog("The humanoid was told to leave the hospital...")
-        if class.is(self, Staff) then
-          self:fire()
-        else
-          -- Set these variables to increase the likelihood of the humanoid managing to get out of the hospital.
-          self.going_home = false
-          self.hospital = self.world:getLocalPlayerHospital()
-          self:goHome("kicked")
-        end
-      end,
-      nil -- Do nothing on cancel
-    ))
     action = self.action_queue[1]
-
+    assert(action)
   end
-  ---- There is an action to start ----
+
   -- Call the action start handler
   TheApp.humanoid_actions[action.name](action, self)
 
@@ -574,7 +523,8 @@ function Humanoid:setNextAction(action, high_priority)
       if removed.is_entering then
         local dest_room = self.world:getRoom(removed.x, removed.y)
         self:unexpectFromRoom(dest_room)
-        if dest_room and removed.reserve_on_resume and removed.reserve_on_resume:isReservedFor(self) then
+        if dest_room and removed.reserve_on_resume and
+            removed.reserve_on_resume:isReservedFor(self) then
           removed.reserve_on_resume:removeReservedUser(self)
           dest_room:tryAdvanceQueue()
         end
@@ -599,7 +549,7 @@ function Humanoid:setNextAction(action, high_priority)
     end
   else
     -- Start the action if it has become the current action
-    Humanoid_startAction(self)
+    self:startAction()
   end
   return self
 end
@@ -608,7 +558,7 @@ function Humanoid:queueAction(action, pos)
   if pos then
     table.insert(self.action_queue, pos + 1, action)
     if pos == 0 then
-      Humanoid_startAction(self)
+      self:startAction()
     end
   else
     self.action_queue[#self.action_queue + 1] = action
@@ -624,7 +574,7 @@ function Humanoid:finishAction(action)
   -- Save the previous action just a while longer.
   self.previous_action = self.action_queue[1]
   table.remove(self.action_queue, 1)
-  Humanoid_startAction(self)
+  self:startAction()
 end
 
 -- Check if the humanoid is running actions intended to leave the room, as indicated by the flag
@@ -642,6 +592,59 @@ function Humanoid:hasLeavingAction()
   return false
 end
 
+--! Handle an empty action queue in some way instead of crashing.
+function Humanoid:_handleEmptyActionQueue()
+  -- if this is a patient that is going home, an empty
+  -- action queue is not a problem
+  if class.is(self, Patient) and self.going_home then
+    return
+  end
+
+  -- First find out if this humanoid is in a room.
+  local room = self:getRoom()
+  if room then
+    room:makeHumanoidLeave(self)
+  end
+
+  -- Give the humanoid an action to avoid crashing.
+  if class.is(self, Staff) then
+    self:queueAction(MeanderAction())
+  elseif class.is(self, GrimReaper) then
+    self:queueAction(IdleAction())
+  else
+    self:queueAction(SeekReceptionAction())
+  end
+
+  -- Open the dialog of the humanoid as feedback to the user.
+  local ui = self.world.ui
+  if class.is(self, Patient) then
+    ui:addWindow(UIPatient(ui, self))
+  elseif class.is(self, Staff) then
+    ui:addWindow(UIStaff(ui, self))
+  end
+
+  -- Tell the player what just happened.
+  self.world:gameLog("")
+  self.world:gameLog("Empty action queue!")
+  self.world:gameLog("Last action: " .. self.previous_action.name)
+  self.world:gameLog(debug.traceback())
+
+  ui:addWindow(UIConfirmDialog(ui, true, {_S.errors.dialog_empty_queue},
+    --[[persistable:humanoid_leave_hospital]] function()
+      self.world:gameLog("The humanoid was told to leave the hospital...")
+      if class.is(self, Staff) then
+        self:fire()
+      else
+        -- Set these variables to increase the likelihood of the humanoid managing to get out of the hospital.
+        self.going_home = false
+        self.hospital = self.world:getLocalPlayerHospital()
+        self:goHome("kicked")
+      end
+    end,
+    nil -- Do nothing on cancel
+  ))
+end
+
 function Humanoid:setType(humanoid_class)
   assert(walk_animations[humanoid_class], "Invalid humanoid class: " .. tostring(humanoid_class))
   self.walk_anims = walk_animations[humanoid_class]
@@ -687,10 +690,10 @@ end
 
 -- Stub functions for handling fatigue. These are overridden by the staff subclass,
 -- but also defined here, so we can just call it on any humanoid
-function Humanoid:tire(amount)
+function Humanoid:tire(amount) -- luacheck: ignore 212 keep args for child class
 end
 
-function Humanoid:wake(amount)
+function Humanoid:wake(amount) -- luacheck: ignore 212 keep args for child class
 end
 
 function Humanoid:updateSpeed()
@@ -913,7 +916,7 @@ end
 !return (string)
 ]]
 function Humanoid:tostring()
-  local name = self.profile and self.profile.name or nil
+  local name = self.profile and self.profile:getFullName() or nil
   local class = self.humanoid_class and self.humanoid_class or "N/A"
   local full_name = "humanoid"
   if (name) then
