@@ -83,6 +83,7 @@ end
 function Patient:setDisease(disease)
   self.disease = disease
   disease.initPatient(self)
+  self.th:setPatientEffect(disease.effect or AnimationEffect.None)
   self.diagnosed = false
   self.diagnosis_progress = 0
   self.cure_rooms_visited = 0
@@ -92,8 +93,6 @@ function Patient:setDisease(disease)
   for i, room in ipairs(self.disease.diagnosis_rooms) do
     self.available_diagnosis_rooms[i] = room
   end
-  -- Decide an insurance company, one out of four patients have one.
-  -- TODO: May need some balancing, but it is roughly the same as in TH.
   local company = math.random(1,12)
   if company < 4 then
     self.insurance_company = company
@@ -269,6 +268,9 @@ function Patient:treatDisease()
 
   hospital:receiveMoneyForTreatment(self)
 
+  -- Remove visual effects of disease.
+  self.th:setPatientEffect(AnimationEffect.None)
+
   -- Either the patient is no longer sick, or he/she dies.
   if self:isTreatmentEffective() then
     self:cure()
@@ -301,8 +303,16 @@ function Patient:isTreatmentEffective()
   local cure_chance = self.hospital.disease_casebook[self.disease.id].cure_effectiveness
   cure_chance = cure_chance * self.diagnosis_progress
 
-  local die = self.die_anims and math.random(1, 100) > cure_chance
-  return not die
+  -- Service quality has a factor on cure chance
+  local room = self:getRoom()
+  local min_impact = 20
+  local service_base = math.max(100 - cure_chance, min_impact)
+
+  local scale = 0.2 -- Quality scaled to +-10%
+  local service_factor = (room:getStaffServiceQuality() - 0.5) * scale
+  cure_chance = cure_chance + (service_base * service_factor)
+
+  return (cure_chance >= math.random(1,100))
 end
 
 --! Change patient internal state to "cured".
@@ -312,21 +322,11 @@ function Patient:cure()
   self.attributes["health"] = 1
 end
 
+--! Patient died, process the event.
 function Patient:die()
   -- It may happen that this patient was just cured and then the room blew up.
-  local hospital = self.hospital
-
-  if hospital.num_deaths < 1 then
-    self.world.ui.adviser:say(_A.information.first_death)
-  end
-  hospital:humanoidDeath(self)
-  if not self.is_debug then
-    local casebook = hospital.disease_casebook[self.disease.id]
-    casebook.fatalities = casebook.fatalities + 1
-  end
-  hospital:msgKilled()
+  self.hospital:humanoidDeath(self)
   self:setMood("dead", "activate")
-  self.world.ui:playSound("boo.wav") -- this sound is always heard
 
   -- Remove any messages and/or callbacks related to the patient.
   self:unregisterCallbacks()
@@ -337,17 +337,23 @@ function Patient:die()
   else
     self:setNextAction(MeanderAction():setCount(1))
   end
-  if self.is_emergency then
-    hospital.emergency.killed_emergency_patients = hospital.emergency.killed_emergency_patients + 1
-  end
   self:queueAction(DieAction())
-  self:updateDynamicInfo(_S.dynamic_info.patient.actions.dying)
+  self:setDynamicInfoText(_S.dynamic_info.patient.actions.dying)
 end
 
+-- Actions we can interrupt for in the canPeeOrPuke function
+local good_actions = {walk=true, idle=true, seek_room=true, queue=true}
+
 function Patient:canPeeOrPuke(current)
-  return ((current.name == "walk" or current.name == "idle" or
-           current.name == "seek_room" or current.name == "queue") and
-      not self.going_home and self.world.map.th:getCellFlags(self.tile_x, self.tile_y).buildable)
+  if not good_actions[current.name] then return false end
+  if self.going_home then return false end
+
+  local th = self.world.map.th
+  local cell_flags = th:getCellFlags(self.tile_x, self.tile_y)
+  if not cell_flags.buildable or not cell_flags.hospital then return false end
+
+  local parcel = cell_flags.parcelId
+  return parcel ~= 0 and th:getPlotOwner(parcel) == self.hospital:getPlayerIndex()
 end
 
 --! Animations for when there is an earth quake
@@ -386,9 +392,7 @@ function Patient:fallingAnnounce()
   (_A.warnings.falling_5),
   (_A.warnings.falling_6),
   }
-  if msg then
-    self.world.ui.adviser:say(msg[math.random(1, #msg)])
-  end
+  self.hospital:giveAdvice(msg)
 end
 
 --! Perform 'shake fist' action.
@@ -421,7 +425,7 @@ function Patient:pee()
     self:changeAttribute("happiness", -0.02) -- not being able to find a loo and doing it in the corridor will make you sad too
     if not self.hospital.did_it_on_floor then
       self.hospital.did_it_on_floor = true
-      self.world.ui.adviser:say(_A.warnings.people_did_it_on_the_floor)
+      self.hospital:giveAdvice({_A.warnings.people_did_it_on_the_floor})
     end
   else
     return
@@ -472,28 +476,26 @@ function Patient:goHome(reason, disease_id)
   if reason == "cured" then
     self:setMood("cured", "activate")
     self:changeAttribute("happiness", 0.8)
-    self.world.ui:playSound("cheer.wav") -- This sound is always heard
-    self.hospital:updateCuredCounts(self)
-    self:updateDynamicInfo(_S.dynamic_info.patient.actions.cured)
-    self.hospital:msgCured()
+    hosp:updateCuredCounts(self)
+    self:setDynamicInfoText(_S.dynamic_info.patient.actions.cured)
 
   elseif reason == "kicked" then
     self:setMood("exit", "activate")
-    self.hospital:updateNotCuredCounts(self, reason)
+    hosp:updateNotCuredCounts(self, reason)
 
   elseif reason == "over_priced" then
     self:setMood("sad_money", "activate")
     self:changeAttribute("happiness", -0.5)
 
     local treatment_name = self.hospital.disease_casebook[disease_id].disease.name
-    self.world.ui.adviser:say(_A.warnings.patient_not_paying:format(treatment_name))
-    self.hospital:updateNotCuredCounts(self, reason)
+    self.hospital:giveAdvice({_A.warnings.patient_not_paying:format(treatment_name)})
+    hosp:updateNotCuredCounts(self, reason)
     self:clearDynamicInfo()
-    self:updateDynamicInfo(_S.dynamic_info.patient.actions.prices_too_high)
+    self:setDynamicInfo('text', {"", _S.dynamic_info.patient.actions.prices_too_high})
 
   elseif reason == "evacuated" then
     self:clearDynamicInfo()
-    self:setDynamicInfo('text', {_S.dynamic_info.patient.actions.epidemic_sent_home})
+    self:setDynamicInfo('text', {"", _S.dynamic_info.patient.actions.epidemic_sent_home})
     self:setMood("exit","activate")
 
   else
@@ -516,6 +518,11 @@ function Patient:goHome(reason, disease_id)
     self.world.dispatcher:dropFromQueue(self)
   end
 
+  -- allow timer to end early and after going_home is set
+  if self.is_emergency then
+    hosp:checkEmergencyOver()
+  end
+
   local room = self:getRoom()
   if room then
     room:makeHumanoidLeave(self)
@@ -529,6 +536,14 @@ function Patient:despawn()
   Humanoid.despawn(self)
 end
 
+-- Checks if the room the patient is in is the final treatment room for their disease
+-- NB: Check diagnosed status as some illnesses use Psych for diagnosis and treatment
+function Patient:_checkIfCureRoom(room)
+  local num_rooms = #self.disease.treatment_rooms
+  return (room and self.disease.treatment_rooms[num_rooms] == room.room_info.id
+      and self.diagnosed)
+end
+
 -- This function handles changing of the different attributes of the patient.
 -- For example if thirst gets over a certain level (now: 0.7), the patient
 -- tries to find a drinks machine nearby.
@@ -540,10 +555,10 @@ function Patient:tickDay()
       self:goHome("kicked")
       if self.diagnosed then
         -- No treatment rooms
-        self:updateDynamicInfo(_S.dynamic_info.patient.actions.no_treatment_available)
+        self:setDynamicInfoText(_S.dynamic_info.patient.actions.no_treatment_available)
       else
         -- No diagnosis rooms
-        self:updateDynamicInfo(_S.dynamic_info.patient.actions.no_diagnoses_available)
+        self:setDynamicInfoText(_S.dynamic_info.patient.actions.no_diagnoses_available)
       end
     elseif self.waiting == 10 then
       self:tapFoot()
@@ -569,20 +584,22 @@ function Patient:tickDay()
   if not Humanoid.tickDay(self) then
     return
   end
+
   -- Die before we poo or drink
-  -- patient has been in the hospital for over 6 months and is still not well, so will become sad and will either get fed up and leave
-  -- or stay in the hope that you will cure them before they die
-  -- strange, but in TH happiness does not go down, even when close to death IMO that is wrong as you would be unhappy if you waited too long.
-  -- TODO death animation for slack female is missing its head. For now the only option is for her to get fed up and leave
-  -- this can be changed when the animation thing is resolved
+  -- Patient has been in the hospital for over 6 months and is still not well, so
+  -- will become sad and will either get fed up and leave or stay in the hope that you
+  -- will cure them before they die.
+  -- Note: This behaviour intentionally differs from Theme Hospital where being close to
+  -- death does not impact happiness.
   -- TODO clean up this block, nonmagical numbers
   if self.attributes["health"] >= 0.18 and self.attributes["health"] < 0.22 then
     self:setMood("sad2", "activate")
     self:changeAttribute("happiness", -0.0002) -- waiting too long will make you sad
-    -- There is a 1/3 chance that the patient will get fed up and leave
-    -- note, this is potentially run 10 ((0.22-0.18)/0.004) times, hence the 1/30 chance.
-    if math.random(1,30) == 1 then
-      self:updateDynamicInfo(_S.dynamic_info.patient.actions.fed_up)
+    -- There is a 1/3 chance that the patient will get fed up and leave.
+    -- This is potentially run 10 ((0.22-0.18)/0.004) times, hence the 1/30 chance.
+    -- If patient is already in the cure rodwom, let the treatment happen.
+    if not self:_checkIfCureRoom(self:getRoom()) and math.random(1,30) == 1 then
+      self:setDynamicInfoText(_S.dynamic_info.patient.actions.fed_up)
       self:setMood("sad2", "deactivate")
       self:goHome("kicked")
     end
@@ -601,20 +618,17 @@ function Patient:tickDay()
   elseif self.attributes["health"] >= 0.01 and self.attributes["health"] < 0.06 then
     self:setMood("sad5", "deactivate")
     self:setMood("sad6", "activate")
-  -- its not looking good
+  -- it's not looking good
   elseif self.attributes["health"] > 0.00 and self.attributes["health"] < 0.01 then
     self.attributes["health"] = 0.0
   -- is there time to say a prayer
   elseif self.attributes["health"] == 0.0 then
-    -- people who are in a room should not die:
-    -- 1. they are being cured in this moment. dying in the last few seconds
-    --    before the cure makes only a subtle difference for gameplay
-    -- 2. they will leave the room soon (toilets, diagnostics) and will die then
     if not self:getRoom() and not self:getCurrentAction().is_leaving then
       self:setMood("sad6", "deactivate")
       self:die()
     end
-    --dead people aren't thirsty
+    -- Patient died, will die when the leave the room, will be cured, or is leaving
+    -- the hospital. Regardless we do not need to adjust any other attribute
     return
   end
 
@@ -691,17 +705,12 @@ function Patient:tickDay()
   -- It is nice to see plants, but dead plants make you unhappy
   self.world:findObjectNear(self, "plant", 2, function(x, y)
     local plant = self.world:getObject(x, y, "plant")
-  if not plant then
-    return
-  end
-    if plant:isPleasing() then
-      self:changeAttribute("happiness", 0.0002)
-    else
-      self:changeAttribute("happiness", -0.0002)
+    if plant then
+      self:changeAttribute("happiness", -0.0003 + (plant:isPleasingFactor() * 0.0001))
     end
   end)
   -- It always makes you happy to see you are in safe place
-  self.world:findObjectNear(self, "extinguisher", 2, function(x, y)
+  self.world:findObjectNear(self, "extinguisher", 2, function()
     self:changeAttribute("happiness", 0.0002)
   end)
   -- sitting makes you happy whilst standing and walking does not
@@ -883,8 +892,8 @@ function Patient:setTile(x, y)
           self.hospital.hospital_littered = true
 
           -- A callout is only needed if there are no handymen employed
-          if not self.hospital:hasStaffOfCategory("Handyman") then
-            self.world.ui.adviser:say(_A.staff_advice.need_handyman_litter)
+          if self.hospital:countStaffOfCategory("Handyman", 1) == 0 then
+            self.hospital:giveAdvice({_A.staff_advice.need_handyman_litter})
           end
         end
       end
@@ -932,17 +941,17 @@ function Patient:addToTreatmentHistory(room)
   end
 end
 
-function Patient:updateDynamicInfo(action_string)
-  -- Retain the old text if only an update is wanted, i.e. no new string is supplied.
-  if action_string == nil then
-    if self.action_string then
-      action_string = self.action_string
-    else
-      action_string = ""
-    end
-  else
-    self.action_string = action_string
-  end
+--! Sets the dynamic text currently in use by the patient
+--!param action_string (string) What text to say in dynamic text
+function Patient:setDynamicInfoText(action_string)
+  self.action_string = action_string
+  self:updateDynamicInfo()
+end
+
+--! Updates a patient's dynamic info
+--! Can be called direct to refresh only
+function Patient:updateDynamicInfo()
+  local action_string = self.action_string or ""
   local info = ""
   if self.going_home then
     self:setDynamicInfo('progress', nil)
@@ -951,7 +960,11 @@ function Patient:updateDynamicInfo(action_string)
       -- The cure was guessed
       info = _S.dynamic_info.patient.guessed_diagnosis:format(self.disease.name)
     else
-      info = _S.dynamic_info.patient.diagnosed:format(self.disease.name)
+      if self.is_emergency then
+        info = _S.dynamic_info.patient.emergency:format(self.disease.name)
+      else
+        info = _S.dynamic_info.patient.diagnosed:format(self.disease.name)
+      end
     end
     self:setDynamicInfo('progress', nil)
   else
@@ -975,6 +988,8 @@ function Patient:updateDynamicInfo(action_string)
     elseif self.vaccinated then
       self:setDynamicInfo('text',
         {action_string, _S.dynamic_info.patient.actions.epidemic_vaccinated, info})
+    else
+      self:setDynamicInfo('text', {action_string, "", info})
     end
   else
     self:setDynamicInfo('text', {action_string, "", info})
@@ -999,8 +1014,10 @@ function Patient:updateMessage(choice)
       -- enable only if research department is built and a room in the treatment chain is undiscovered
       local req = self.hospital:checkDiseaseRequirements(self.disease.id)
       if req then
+        enabled = (self.hospital:countRoomOfType("research", 1) > 0 and
+                   self.hospital:countStaffOfCategory("Researcher", 1) > 0)
+
         local strings = _S.fax.disease_discovered_patient_choice
-        enabled = self.hospital:hasRoomOfType("research") and self.hospital:hasStaffOfCategory("Researcher")
         local output_text = strings.can_not_cure
         if #req.rooms == 1 then
           local room_name, required_staff, staff_name = self.world:getRoomNameAndRequiredStaffName(req.rooms[1])
@@ -1010,13 +1027,7 @@ function Patient:updateMessage(choice)
             output_text = strings.need_to_build:format(room_name)
           end
         elseif #req.rooms == 0 and next(req.staff) then
-          local staffclass_to_string = {
-              Nurse        = _S.staff_title.nurse,
-              Doctor       = _S.staff_title.doctor,
-              Surgeon      = _S.staff_title.surgeon,
-              Psychiatrist = _S.staff_title.psychiatrist,
-          }
-          output_text = strings.need_to_employ:format(staffclass_to_string[next(req.staff)])
+          output_text = strings.need_to_employ:format(StaffProfile.translateStaffClass(next(req.staff)))
         end
         self.message[3].text = output_text
       else
@@ -1094,6 +1105,16 @@ function Patient:afterLoad(old, new)
       self.going_to_toilet = "no"
     end
   end
+  if old < 158 then
+    -- Always overwrite patient effect from old saves since it is
+    -- loaded from a dummy field which used to have another meaning.
+    if self.disease and not self.cured then
+      self.th:setPatientEffect(self.disease.effect or AnimationEffect.None)
+    else
+      self.th:setPatientEffect(AnimationEffect.None)
+    end
+  end
+  self:updateDynamicInfo()
   Humanoid.afterLoad(self, old, new)
 end
 
@@ -1112,39 +1133,36 @@ end
 !param meander_before_resume (bool) Indicator to insert an additional action before resuming.
 ]]
 function Patient:interruptAndRequeueAction(current_action, queue_pos, meander_before_resume)
-  if current_action.name == "idle" or current_action.name == "walk" then
-    local requeue_action = {
-        name = current_action.name,
-        x = current_action.x,
-        y = current_action.y,
-        must_happen = current_action.must_happen,
-        is_entering = current_action.is_entering,
-    }
-    self:queueAction(requeue_action, queue_pos)
-    -- If we were idling, also go away a little before continuing with
-    -- that important action.
-    if current_action.name == "idle" and meander_before_resume then
+  -- Rebuild the current action.
+  local requeue_action
+  if current_action.name == "idle" then
+    requeue_action = IdleAction():setMustHappen(current_action.must_happen)
+    requeue_action:setDirection(current_action.direction)
+
+    -- Go away a little before continuing with that important action.
+    if meander_before_resume then
       self:queueAction(MeanderAction():setCount(1), queue_pos)
-    -- if walking, and action was already started, use the original must_happen
-    elseif current_action.name == "walk" then
-      if current_action.saved_must_happen ~= nil then
-        requeue_action.must_happen = current_action.saved_must_happen
-      end
-      -- need to copy the reserve_on_resume, otherwise the new queued action will not
-      -- unreserve on interrupt
-      requeue_action.reserve_on_resume = current_action.reserve_on_resume
     end
+  elseif current_action.name == "walk" then
+    requeue_action = WalkAction(current_action.x, current_action.y)
+    requeue_action:setMustHappen(current_action.must_happen)
+
+    -- need to copy the reserve_on_resume, otherwise the new queued action will not
+    -- unreserve on interrupt
+    requeue_action.reserve_on_resume = current_action.reserve_on_resume
+    requeue_action:setIsEntering(current_action.is_entering)
   else
     -- We were seeking a room, start that action from the beginning
     -- i.e. do not set the must_happen flag.
-    self:queueAction({
-      name = current_action.name,
-      room_type = current_action.room_type,
-      message_sent = true,
-      diagnosis_room = current_action.diagnosis_room,
-      treatment_room = current_action.treatment_room,
-    }, queue_pos)
+    requeue_action = SeekRoomAction(current_action.room_type)
+    if current_action.treatment_room then requeue_action:enableTreatmentRoom() end
+    if current_action.diagnosis_room then
+      requeue_action:setDiagnosisRoom(current_action.diagnosis_room)
+    end
+    requeue_action.message_sent = true
   end
+  self:queueAction(requeue_action, queue_pos)
+
   -- now interrupt
   if current_action.on_interrupt then
     current_action.on_interrupt(current_action, self)
@@ -1154,4 +1172,4 @@ function Patient:interruptAndRequeueAction(current_action, queue_pos, meander_be
 end
 
 -- Dummy callback for savegame compatibility
-local callbackNewRoom = --[[persistable:patient_toilet_build_callback]] function(room) end
+local callbackNewRoom = --[[persistable:patient_toilet_build_callback]] function() end
