@@ -107,7 +107,7 @@ function GameUI:setupGlobalKeyHandlers()
      [tostring(self.app.hotkeys["ingame_scroll_up"])] = {x = 0, y = -10},
      [tostring(self.app.hotkeys["ingame_scroll_down"])] = {x = 0, y = 10},
      [tostring(self.app.hotkeys["ingame_scroll_left"])] = {x = -10, y = 0},
-     [tostring(self.app.hotkeys["ingame_scroll_right"])]	= {x = 10, y = 0},
+     [tostring(self.app.hotkeys["ingame_scroll_right"])] = {x = 10, y = 0},
   }
 
   -- This is the long version of the shift speed key.
@@ -244,7 +244,7 @@ function GameUI:draw(canvas)
   local dy = self.screen_offset_y +
       math.floor((0.5 - math.random()) * self.shake_screen_intensity * shake_screen_max_movement * 2)
   if canvas:scale(zoom) then
-    app.map:draw(canvas, dx, dy, math.floor(config.width / zoom), math.floor(config.height / zoom), 0, 0)
+    app.map:draw(canvas, dx, dy, math.ceil(config.width / zoom), math.ceil(config.height / zoom), 0, 0)
     canvas:scale(1)
   else
     self:setZoom(1)
@@ -608,7 +608,8 @@ function GameUI:onMouseMove(x, y, dx, dy)
     --map.th:setCell(highlight_x, highlight_y, 4, 0)
     highlight_x = nil
   end
-  if 1 <= wx and wx <= 128 and 1 <= wy and wy <= 128 then
+  local map_width, map_height = map.th:size()
+  if 1 <= wx and wx <= map_width and 1 <= wy and wy <= map_height then
     if map.th:getCellFlags(wx, wy).passable then
       --map.th:setCell(wx, wy, 4, 24 + 8 * 256)
       highlight_x = wx
@@ -643,10 +644,27 @@ function GameUI:onMouseUp(code, x, y)
     else -- No room chosen yet, but about to edit one.
       if button == "left" then -- Take the clicked one.
         local room = self.app.world:getRoom(self:ScreenToWorld(x, y))
-        if room and not room.crashed then
-          self:setCursor(self.waiting_cursor)
-          self.edit_room = room
-          room:tryToEdit()
+        if room then
+          if not room.crashed then
+            self:setCursor(self.waiting_cursor)
+            self.edit_room = room
+            room:tryToEdit()
+          else
+            if self.app.config.remove_destroyed_rooms then
+              local room_cost = room:calculateRemovalCost()
+              self:setEditRoom(false)
+              -- show confirmation dialog for removing the room
+              self:addWindow(UIConfirmDialog(self, false, _S.confirmation.remove_destroyed_room:format(room_cost),
+              --[[persistable:remove_destroyed_room_confirm_dialog]]function()
+                local world = room.world
+                UIEditRoom:removeRoom(false, room, world)
+                world:resetSideObjects()
+                world.rooms[room.id] = nil
+                self.hospital:spendMoney(room_cost, _S.transactions.remove_room)
+                end
+              ))
+            end
+          end
         end
       else -- right click, we don't want to edit a room after all.
         self:setEditRoom(false)
@@ -682,7 +700,7 @@ end
 --!param x (float) normalised x value of the gesture
 --!param y (float) normalised y value of the gesture
 --!return (boolean) event processed indicator
-function GameUI:onMultiGesture(numfingers, dTheta, dDist, x, y)
+function GameUI:onMultiGesture(numfingers, dTheta, dDist, x, y) -- luacheck: ignore 212 dTheta
   -- only deal with 2 finger events for now
   if numfingers == 2 then
     -- calculate magnitude of pinch
@@ -732,6 +750,27 @@ function GameUI:onMouseWheel(x, y)
   return UI.onMouseWheel(self, x, y)
 end
 
+--! Announcements to the player.
+--!param msgs (array of string) Messages to select from.
+--!param priority (optional valid AnnouncementPriority selection) Priority of announcement
+--!param chance_to_play (optional float in range (0, 1]) Fraction of times that the
+--    call actually says something.
+--!return (boolean) Whether a message was given to the user.
+function GameUI:playRandomAnnouncement(msgs, priority, chance_to_play, played_callback, played_callback_delay)
+  local max_rnd = #msgs
+  if chance_to_play and chance_to_play > 0 and chance_to_play < 1 then
+    -- Scale by the fraction.
+    max_rnd = math.floor(max_rnd / chance_to_play)
+  end
+
+  local index = (max_rnd == 1) and 1 or math.random(1, max_rnd)
+  if index <= #msgs then
+    self:playAnnouncement(msgs[index], priority, played_callback, played_callback_delay)
+    return true
+  end
+  return false
+end
+
 function GameUI:playAnnouncement(name, priority, played_callback, played_callback_delay)
   self.announcer:playAnnouncement(name, priority, played_callback, played_callback_delay)
 end
@@ -829,7 +868,7 @@ function GameUI.limitPointToDiamond(dx, dy, visible_diamond, do_limit)
     if do_limit then
       -- Determine the quadrant which the point lies in and accordingly set:
       --  (vx, vy) : a unit vector perpendicular to the diamond edge in the quadrant
-      --  (p1x, p1y), (p2x, p2y) : the two diamond verticies in the quadrant
+      --  (p1x, p1y), (p2x, p2y) : the two diamond vertices in the quadrant
       --  d : distance from the point to the line defined by the diamond edge (not the line segment itself)
       local vx, vy, d
       local p1x, p1y, p2x, p2y = 0, 0, 0, 0
@@ -929,7 +968,7 @@ function UI:togglePlayAnnouncements()
   self.app:saveConfig()
 end
 
-function UI:togglePlayMusic(item)
+function UI:togglePlayMusic()
   if not self.app.audio.background_music then
     self.app.config.play_music = true
     self.app.audio:playRandomBackgroundTrack() -- play
@@ -1060,7 +1099,7 @@ tutorial_phases = {
   },
 }
 end
-tutorial_phases = setmetatable({}, {__index = function(t, k)
+tutorial_phases = setmetatable({}, {__index = function(_, k)
   make_tutorial_phases()
   return tutorial_phases[k]
 end})
@@ -1131,15 +1170,15 @@ function GameUI:startTutorial(chapter)
   self:tutorialStep(chapter, 0, 1)
 end
 
---! Converts centre of screen co-ordinates to world tile positions and stores the values for later recall
--- param index (integer) Postion in recallpositions table
+--! Converts centre of screen coordinates to world tile positions and stores the values for later recall
+-- param index (integer) Position in recallpositions table
 function GameUI:setMapRecallPosition(index)
   local cx, cy = self:ScreenToWorld(self.app.config.width / 2, self.app.config.height / 2)
   self.recallpositions[index] = {x = cx, y = cy, z = self.zoom_factor}
 end
 
---! Retrives stored recall position and attempts to scroll to that position - will be limited to the bounds of the camera when zoomed out
--- param index (integer) Postion in recallpositions table
+--! Retrieves stored recall position and attempts to scroll to that position - will be limited to the bounds of the camera when zoomed out
+-- param index (integer) Position in recallpositions table
 function GameUI:recallMapPosition(index)
   if self.recallpositions[index] ~= nil then
     local sx, sy = self.app.map:WorldToScreen(self.recallpositions[index].x,  self.recallpositions[index].y)
@@ -1158,7 +1197,7 @@ function GameUI:setEditRoom(enabled)
     -- If the actual editing hasn't started yet but is on its way,
     -- activate the room again.
     if class.is(self.edit_room, Room) and self.cursor == self.waiting_cursor then
-      self.edit_room.is_active = true
+      self.app.world:markRoomAsBuilt(self.edit_room)
     else
       -- If we are currently editing a room it may happen that we need to abort it.
       -- Also remove any dialog where the user is buying items.
@@ -1211,6 +1250,8 @@ function GameUI:afterLoad(old, new)
     self.announcer = Announcer(self.app)
   end
 
+  self.announcer.playing = false
+
   return UI.afterLoad(self, old, new)
 end
 
@@ -1228,7 +1269,7 @@ end
 --! Offers a confirmation window to quit the game and return to main menu
 -- NB: overrides UI.quit, do NOT call it from here
 function GameUI:quit()
-  self:addWindow(UIConfirmDialog(self, _S.confirmation.quit, --[[persistable:gameui_confirm_quit]] function()
+  self:addWindow(UIConfirmDialog(self, false, _S.confirmation.quit, --[[persistable:gameui_confirm_quit]] function()
     self.app:loadMainMenu()
   end))
 end
