@@ -105,7 +105,7 @@ local action_seek_room_goto_room = permanent"action_seek_room_goto_room"( functi
   humanoid.message_callback = nil
   humanoid:setNextAction(room:createEnterAction(humanoid))
   humanoid.next_room_to_visit = room
-  humanoid:updateDynamicInfo(_S.dynamic_info.patient.actions.on_my_way_to
+  humanoid:setDynamicInfoText(_S.dynamic_info.patient.actions.on_my_way_to
     :format(room.room_info.name))
   room.door:updateDynamicInfo()
   if not room:testStaffCriteria(room:getRequiredStaffCriteria()) then
@@ -117,61 +117,6 @@ local action_seek_room_goto_room = permanent"action_seek_room_goto_room"( functi
   end
 end)
 
-local function action_seek_room_no_treatment_room_found(room_type, humanoid)
-  -- Emergency patients also don't need to ask what to do, they'll just wait for the player
-  -- to build the necessary room.
-  if humanoid.is_emergency then
-    return
-  end
-  -- Wait two months before going home anyway.
-  humanoid.waiting = 60
-  local strings = _S.fax.disease_discovered_patient_choice
-  -- Can this room be built right now? What is then missing?
-
-  local output_text = strings.can_not_cure
-  -- TODO: can we really assert this? Or should we just make the patient go home?
-  --local room = assert(humanoid.world.available_rooms[room_type], "room " .. room_type .. " not available")
-
-  local req = humanoid.hospital:checkDiseaseRequirements(humanoid.disease.id)
-  local research_enabled = false
-  if req then
-    research_enabled = (humanoid.hospital:countRoomOfType("research", 1) > 0 and
-                        humanoid.hospital:countStaffOfCategory("Researcher", 1) > 0)
-    if #req.rooms == 1 then
-      local room_name, required_staff, staff_name = humanoid.world:getRoomNameAndRequiredStaffName(req.rooms[1])
-      if req.staff[required_staff] or 0 > 0 then
-        output_text = strings.need_to_build_and_employ:format(room_name, staff_name)
-      else
-        output_text = strings.need_to_build:format(room_name)
-      end
-    elseif #req.rooms == 0 and next(req.staff) then
-      local staffclass_to_string = {
-          Nurse        = _S.staff_title.nurse,
-          Doctor       = _S.staff_title.doctor,
-          Surgeon      = _S.staff_title.surgeon,
-          Psychiatrist = _S.staff_title.psychiatrist,
-      }
-      output_text = strings.need_to_employ:format(staffclass_to_string[next(req.staff)])
-    end
-  end
-
-  local message = {
-    {text = strings.disease_name:format(humanoid.disease.name)},
-    {text = " "},
-    {text = output_text},
-    {text = strings.what_to_do_question},
-    choices = {
-      {text = strings.choices.send_home, choice = "send_home"},
-      {text = strings.choices.wait,      choice = "wait"},
-      {text = strings.choices.research,  choice = "research", enabled = research_enabled},
-    },
-  }
-  -- Ok, send the message in all channels.
-  TheApp.ui.bottom_panel:queueMessage("information", message, humanoid)
-  humanoid:setMood("patient_wait", "activate")
-  humanoid:updateDynamicInfo(_S.dynamic_info.patient.actions.awaiting_decision)
-end
-
 local function action_seek_room_no_diagnosis_room_found(action, humanoid)
    --If it's the VIP then there's been an explosion. Skip the exploded room
   if humanoid.humanoid_class == "VIP" then
@@ -184,7 +129,8 @@ local function action_seek_room_no_diagnosis_room_found(action, humanoid)
   if humanoid.diagnosis_progress < humanoid.hospital.policies["send_home"] then
     -- Send home automatically
     humanoid:goHome("kicked")
-    humanoid:updateDynamicInfo(_S.dynamic_info.patient.actions.no_diagnoses_available)
+    humanoid:setDynamicInfoText(_S.dynamic_info.patient.actions.no_diagnoses_available)
+
   elseif humanoid.diagnosis_progress < humanoid.hospital.policies["guess_cure"] or
       not humanoid.hospital.disease_casebook[humanoid.disease.id].discovered then
     -- If the disease hasn't been discovered yet it cannot be guessed, go here instead.
@@ -192,23 +138,8 @@ local function action_seek_room_no_diagnosis_room_found(action, humanoid)
     -- Wait two months before going home anyway.
     humanoid:setMood("patient_wait", "activate")
     humanoid.waiting = 60
-    local guess_enabled = humanoid.hospital.disease_casebook[humanoid.disease.id].discovered
-    local message = {
-      {text = _S.fax.diagnosis_failed.situation},
-      {text = " "},
-      {text = _S.fax.diagnosis_failed.what_to_do_question},
-      choices = {
-        {text = _S.fax.diagnosis_failed.choices.send_home,   choice = "send_home"},
-        {text = _S.fax.diagnosis_failed.choices.take_chance, choice = "guess_cure", enabled = guess_enabled},
-        {text = _S.fax.diagnosis_failed.choices.wait,        choice = "wait"},
-      },
-    }
-    if guess_enabled then
-      table.insert(message, 3, {text = _S.fax.diagnosis_failed.partial_diagnosis_percentage_name
-        :format(math.round(humanoid.diagnosis_progress*100), humanoid.disease.name)})
-    end
-    TheApp.ui.bottom_panel:queueMessage("information", message, humanoid)
-    humanoid:updateDynamicInfo(_S.dynamic_info.patient.actions.awaiting_decision)
+    humanoid.hospital:makeNoDiagnosisRoomFax(humanoid)
+    humanoid:setDynamicInfoText(_S.dynamic_info.patient.actions.awaiting_decision)
     -- This seek_room action will be reused, return that it's valid.
     return true
   else
@@ -220,11 +151,9 @@ local function action_seek_room_no_diagnosis_room_found(action, humanoid)
     humanoid:unregisterRoomRemoveCallback(action.remove_callback)
     humanoid:unregisterStaffChangeCallback(action.staff_change_callback)
     if humanoid:agreesToPay(humanoid.disease.id) then
-      humanoid:queueAction({
-        name = "seek_room",
-        room_type = humanoid.disease.treatment_rooms[1],
-        treatment_room = true,
-      }, 1)
+      local seek_action = SeekRoomAction(humanoid.disease.treatment_rooms[1])
+      seek_action:enableTreatmentRoom()
+      humanoid:queueAction(seek_action, 1)
     else
       humanoid:goHome("over_priced", humanoid.disease.id)
     end
@@ -264,9 +193,7 @@ local function action_seek_room_start(action, humanoid)
     local room = action_seek_room_find_room(action, humanoid)
     -- if we have the room but not the staff we shouldn't seek out the room either
     if room then
-      if humanoid.message then
-        TheApp.ui.bottom_panel:removeMessage(humanoid)
-      end
+      humanoid.hospital:removeMessage(humanoid)
       action_seek_room_goto_room(room, humanoid, action.diagnosis_room)
       return
     end
@@ -278,9 +205,7 @@ local function action_seek_room_start(action, humanoid)
     local room = action_seek_room_find_room(action, humanoid)
     -- if we have the room but not the staff we shouldn't seek out the room either
     if room and (not req or not action.treatment_room) then
-      if humanoid.message then
-        TheApp.ui.bottom_panel:removeMessage(humanoid)
-      end
+      humanoid.hospital:removeMessage(humanoid)
       action_seek_room_goto_room(room, humanoid, action.diagnosis_room)
       return
     end
@@ -314,7 +239,7 @@ local function action_seek_room_start(action, humanoid)
       if not staff.fired and not room_req then
         local room = action_seek_room_find_room(action, humanoid)
         if room then
-          TheApp.ui.bottom_panel:removeMessage(humanoid)
+          humanoid.hospital:removeMessage(humanoid)
           humanoid:unregisterRoomBuildCallback(build_callback)
           humanoid:unregisterRoomRemoveCallback(remove_callback)
           humanoid:unregisterStaffChangeCallback(staff_change_callback)
@@ -336,7 +261,7 @@ local function action_seek_room_start(action, humanoid)
         -- So the room that we're going to is not actually the room we waited for to be built.
         -- Example: Will go to ward, but is waiting for the operating theatre.
         -- Clean up and start over to find the room we actually want to go to.
-        TheApp.ui.bottom_panel:removeMessage(humanoid)
+        humanoid.hospital:removeMessage(humanoid)
         humanoid:unregisterRoomBuildCallback(build_callback)
         humanoid:unregisterRoomRemoveCallback(remove_callback)
         humanoid:unregisterStaffChangeCallback(staff_change_callback)
@@ -360,7 +285,7 @@ local function action_seek_room_start(action, humanoid)
         -- get required staff
         if not humanoid.diagnosed or (not room_req or not action.treatment_room) then
           action_seek_room_goto_room(rm, humanoid, action.diagnosis_room)
-          TheApp.ui.bottom_panel:removeMessage(humanoid)
+          humanoid.hospital:removeMessage(humanoid)
           humanoid:unregisterRoomBuildCallback(build_callback)
           humanoid:unregisterRoomRemoveCallback(remove_callback)
           humanoid:unregisterStaffChangeCallback(staff_change_callback)
@@ -382,16 +307,23 @@ local function action_seek_room_start(action, humanoid)
     if not action.message_sent then
       -- Make a message about that something needs to be done about this patient
       if humanoid.diagnosed then
-        -- The patient is diagnosed, a treatment room is missing.
-        -- It may happen that it is another room in a series which is missing.
-        local room_to_find = action.room_type_needed and action.room_type_needed or action.room_type
-        action_seek_room_no_treatment_room_found(room_to_find, humanoid)
+        -- Emergency patients also don't need to ask what to do, they'll just wait for the player
+        -- to build the necessary room.
+        if not humanoid.is_emergency then
+          -- The patient is diagnosed, a treatment room is missing.
+          -- It may happen that it is another room in a series which is missing.
+          humanoid.hospital:makeNoTreatmentRoomFax(humanoid)
+          -- Wait two months before going home anyway.
+          humanoid.waiting = 60
+          humanoid:setMood("patient_wait", "activate")
+          humanoid:setDynamicInfoText(_S.dynamic_info.patient.actions.awaiting_decision)
+        end
       else
         -- No more diagnosis rooms can be found
         -- The GP's office is a special case. TODO: Make a custom message anyway?
         if action.room_type == "gp" then
           humanoid:setMood("patient_wait", "activate")
-          humanoid:updateDynamicInfo(_S.dynamic_info.patient.actions.no_gp_available)
+          humanoid:setDynamicInfoText(_S.dynamic_info.patient.actions.no_gp_available)
         else
           action_still_valid = action_seek_room_no_diagnosis_room_found(action, humanoid)
         end
