@@ -39,6 +39,10 @@ function Map:Map(app)
   self.debug_font = false
   self.debug_tick_timer = 1
   self:setTemperatureDisplayMethod(app.config.warmth_colors_display_default)
+
+  -- Difficulty of the level (string) "easy", "full", "hard".
+  -- Use map:getDifficulty() to query the value.
+  self.difficulty = nil
 end
 
 local flag_cache = {}
@@ -104,10 +108,10 @@ function Map:registerTemperatureDisplayMethod()
   self.th:setTemperatureDisplay(self.temperature_display_method)
 end
 
--- Convert between world co-ordinates and screen co-ordinates
--- World co-ordinates are (at least for standard maps) in the range [1, 128)
+-- Convert between world coordinates and screen coordinates
+-- World coordinates are (at least for standard maps) in the range [1, 128)
 -- for both x and y, with the floor of the values giving the cell index.
--- Screen co-ordinates are pixels relative to the map origin - NOT relative to
+-- Screen coordinates are pixels relative to the map origin - NOT relative to
 -- the top-left corner of the screen (use UI:WorldToScreen and UI:ScreenToWorld
 -- for this).
 
@@ -268,9 +272,23 @@ function Map:load(level, difficulty, level_name, map_file, level_intro, map_edit
     end
   end
 
+  -- fix original level 6 map
+  if level == 6 and not map_file and not map_editor then
+    self:setCellFlags(56, 71, {hospital = true, buildable = true, buildableNorth = true, buildableSouth = true, buildableEast = true, buildableWest = true})
+    self:setCellFlags(58, 72, {passable = false})
+  end
+
   return objects
 end
 
+--! Get the difficulty of the level. Custom levels and campaign always have medium difficulty.
+--!return (int) difficulty of the level, 1=easy, 2=medium, 3=hard.
+function Map:getDifficulty()
+  if self.difficulty == "easy" then return 1 end
+  if self.difficulty == "hard" then return 3 end
+  return 2
+end
+
 --[[! Sets the plot owner of the given plot number to the given new owner. Makes sure
       that any room adjacent to the new plot have walls in all directions after the purchase.
 !param plot_number (int) Number of the plot to change owner of. Plot 0 is the outside and
@@ -778,4 +796,43 @@ function Map:afterLoad(old, new)
     -- Issue #1105 update pathfinding (rebuild walls) potentially broken by side object placement
     self.th:updatePathfinding()
   end
+  if old < 136 then
+    if self.level_number == 6 then
+      self:setCellFlags(56, 71, {hospital = true, buildable = true, buildableNorth = true, buildableSouth = true, buildableEast = true, buildableWest = true})
+      self:setCellFlags(58, 72, {passable = false})
+    end
+  end
+  if old < 161 then
+    -- Permanently fix the 0.65 trophy bug (#2004)
+    -- make sure we erase the hofix variable too
+    self.hotfix1 = nil
+    self.level_config.awards_trophies.TrophyAllCuredBonus = 20000
+    self.level_config.awards_trophies.AllCuresBonus = 5000
+  end
+  if old < 164 then
+    -- New feature, by default non-visual illnesses were always available
+    -- at the start
+    self.level_config.non_visuals_available = {
+    [0] = {Value = 0}, -- I_UNCOMMON_COLD
+    {Value = 0}, -- I_BROKEN_WIND
+    {Value = 0}, -- I_SPARE_RIBS
+    {Value = 0}, -- I_KIDNEY_BEANS
+    {Value = 0}, -- I_BROKEN_HEART
+    {Value = 0}, -- I_RUPTURED_NODULES
+    {Value = 0}, -- I_MULTIPLE_TV_PERSONALITIES
+    {Value = 0}, -- I_INFECTIOUS_LAUGHTER
+    {Value = 0}, -- I_CORRUGATED_ANKLES
+    {Value = 0}, -- I_CHRONIC_NOSEHAIR
+    {Value = 0}, -- I_3RD_DEGREE_SIDEBURNS
+    {Value = 0}, -- I_FAKE_BLOOD
+    {Value = 0}, -- I_GASTRIC_EJECTIONS
+    {Value = 0}, -- I_THE_SQUITS
+    {Value = 0}, -- I_IRON_LUNGS
+    {Value = 0}, -- I_SWEATY_PALMS
+    {Value = 0}, -- I_HEAPED_PILES
+    {Value = 0}, -- I_GUT_ROT
+    {Value = 0}, -- I_GOLF_STONES
+    {Value = 0}, -- I_UNEXPECTED_SWELLING
+    }
+  end
 end
