@@ -23,6 +23,8 @@ class "Room"
 ---@type Room
 local Room = _G["Room"]
 
+local COST_RECOVERY = 0.40 -- Percentage cost recovery of destroyed room items
+
 function Room:Room(x, y, w, h, id, room_info, world, hospital, door, door2)
   self.id = id
   self.world = world
@@ -326,12 +328,12 @@ function Room:onHumanoidEnter(humanoid)
     self:tryAdvanceQueue()
     return
   end
-  local msg = {
+  local researcher_desks = {
     (_A.warnings.researcher_needs_desk_1),
     (_A.warnings.researcher_needs_desk_2),
     (_A.warnings.researcher_needs_desk_3),
   }
-  local msg_nurse = {
+  local nurse_desks = {
     (_A.warnings.nurse_needs_desk_1),
     (_A.warnings.nurse_needs_desk_2),
   }
@@ -343,10 +345,10 @@ function Room:onHumanoidEnter(humanoid)
         local staff_member = self:getStaffMember()
         self.humanoids[humanoid] = true
           if staff_member.profile.is_researcher and self.room_info.id == "research" then
-            self.world.ui.adviser:say(msg[math.random(1, #msg)])
+            self.hospital:giveAdvice(researcher_desks)
           end
           if staff_member.humanoid_class == "Nurse" and self.room_info.id == "ward" then
-            self.world.ui.adviser:say(msg_nurse[math.random(1, #msg_nurse)])
+            self.hospital:giveAdvice(nurse_desks)
           end
         if not staff_member.dealing_with_patient then
           staff_member:setNextAction(self:createLeaveAction())
@@ -427,10 +429,20 @@ function Room:createDealtWithPatientCallback(humanoid)
 end
 
 --! Get the current staff member.
--- Can be overridden in rooms with multiple staff members to return the desired one.
+-- In multi-occupancy rooms this returns the staff member with the minimum service quality
 --!return (staff) The current staff member.
 function Room:getStaffMember()
-  return self.staff_member
+  if not self.staff_member_set then return self.staff_member end
+
+  local staff
+  for staff_member, _ in pairs(self.staff_member_set) do
+    if not staff_member.fired and not staff_member:hasLeavingAction() then
+      if not staff or staff:getServiceQuality() > staff_member:getServiceQuality() then
+        staff = staff_member
+      end
+    end
+  end
+  return staff
 end
 
 --! Set the current staff member.
@@ -571,18 +583,20 @@ function Room:onHumanoidLeave(humanoid)
       humanoid.waiting_on_other_staff = nil
       self.dealt_patient_callback = nil
     end
-    -- Make patients leave the room if there are no longer enough staff
+    -- Make patients leave the room (except wards) if there are no longer enough staff
     if not self:testStaffCriteria(self:getRequiredStaffCriteria()) then
-      local patient_needs_to_reenter = false
+      local call_for_new_staff = self.door.queue:patientSize() > 0
       for room_humanoid in pairs(self.humanoids) do
         if class.is(room_humanoid, Patient) and self:shouldHavePatientReenter(room_humanoid) then
-          self:makeHumanoidLeave(room_humanoid)
-          room_humanoid:queueAction(self:createEnterAction(room_humanoid))
-          patient_needs_to_reenter = true
+          call_for_new_staff = true
+          if self.room_info.id ~= "ward" then
+            self:makeHumanoidLeave(room_humanoid)
+            room_humanoid:queueAction(self:createEnterAction(room_humanoid))
+          end
         end
       end
       -- Call for staff if needed
-      if self.is_active and (patient_needs_to_reenter or self.door.queue:patientSize() > 0) then
+      if self.is_active and call_for_new_staff then
         self.world.dispatcher:callForStaff(self)
       end
     end
@@ -659,7 +673,7 @@ function Room:roomFinished()
   end
   -- Show information about the room if not already shown.
   -- Also only show them if the player is playing the original campaign.
-  if tonumber(self.world.map.level_number) and not self.world.room_information_dialogs_off then
+  if tonumber(self.world.map.level_number) and self.world.room_information_dialogs then
     if not self.world.room_built[self.room_info.id] then
       self.world.ui:addWindow(UIInformation(self.world.ui, _S.room_descriptions[self.room_info.id]))
       self.world.room_built[self.room_info.id] = true
@@ -865,6 +879,9 @@ function Room:crashRoom()
 
   self.hospital.num_explosions = self.hospital.num_explosions + 1
 
+  local value_change = self.hospital.research.research_progress[self.room_info].build_cost
+  self.hospital:changeValue(value_change * -1)
+  self.hospital:changeReputation("room_crash")
   self.crashed = true
   self:deactivate()
 end
@@ -920,7 +937,7 @@ function Room:deactivate()
 
   -- crashRoom might have deactivated the door already
   if self.door.queue then
-    self.door.queue:rerouteAllPatients(SeekRoomAction(self.room_info.id))
+    self.door.queue:rerouteAllPatients(self.room_info.id)
   end
 
   self.hospital:removeRatholesAroundRoom(self)
@@ -1030,3 +1047,36 @@ function Room:getStaffServiceQuality()
 
   return quality
 end
+
+--! Count the number of windows in the room
+--!return (int) Number of windows
+function Room:countWindows()
+  local window_tile = {[116]=true, [117]=true, [118]=true, [119]=true,
+   [124]=true, [125]=true, [126]=true, [127]=true}
+  local map, count = self.world.ui.app.map.th, 0
+  for x = self.x, self.x + self.width do
+    for y = self.y, self.y + self.height do
+      if window_tile[map:getCell(x, y, 2)] or window_tile[map:getCell(x, y, 3)] then
+        count = count + 1
+      end
+    end
+  end
+  return count
+end
+
+--! Get the removal cost
+--!return (int) cost of the room
+function Room:calculateRemovalCost()
+  -- Charge double to clean it up
+  local progress = self.hospital.research.research_progress
+  local cost = math.floor(progress[self.room_info].build_cost * 2)
+
+  -- Recover some cost as scrap
+  for obj, _ in pairs(self.room_info.objects_needed) do
+    -- Get how much this item costs.
+    local obj_cost = self.hospital:getObjectBuildCost(obj)
+    -- recover a percentage of cost as scrap value
+    cost = cost - math.floor(obj_cost * COST_RECOVERY)
+  end
+  return cost
+end
