@@ -247,6 +247,8 @@ function UI:runDebugScript()
   _ = TheApp.ui and TheApp.ui.debug_cursor_entity
   local script = assert(loadfile(lua_dir .. path_sep .. "debug_script.lua"))
   script()
+  -- Clear _ after the script to prevent save corruption
+  _ = nil
 end
 
 function UI:setupGlobalKeyHandlers()
@@ -254,8 +256,9 @@ function UI:setupGlobalKeyHandlers()
   self:addKeyHandler("global_cancel", self, self.closeWindow)
   self:addKeyHandler("global_cancel_alt", self, self.closeWindow)
   self:addKeyHandler("global_stop_movie", self, self.stopMovie)
+  self:addKeyHandler("global_stop_movie_alt", self, self.stopMovie)
   self:addKeyHandler("global_screenshot", self, self.makeScreenshot)
-  self:addKeyHandler("global_fullscreen_toggle", self, self.toggleFullscreen)
+  self:addKeyHandler("global_fullscreen_toggle", self, self.fullscreenHotkey)
   self:addKeyHandler("global_exitApp", self, self.exitApplication)
   self:addKeyHandler("global_resetApp", self, self.resetApp)
   self:addKeyHandler("global_releaseMouse", self, self.releaseMouse)
@@ -361,7 +364,7 @@ end
 --!param ... Additional arguments to `callback`.
 function UI:addKeyHandler(keys, window, callback, ...)
   -- It is necessary to clone the key table into another temporary table, as if we don't the original table that we take it from will lose
-  -- the last key of that table permenently in the next line of code after this one, until the program is restarted.
+  -- the last key of that table permanently in the next line of code after this one, until the program is restarted.
   -- I.E. if the "ingame_quitLevel" hotkey from the "hotkeys_values" table in "config_finder.lua" is a table that looks like this:
   --   {"shift", "q"}
   -- We would lose the "q" element until we restarted the game and the "hotkey.txt" was read from again, causing the "ingame_quitLevel"
@@ -590,6 +593,7 @@ function UI:onChangeResolution()
   for _, window in ipairs(self.windows) do
     window:onChangeResolution()
   end
+  self.app.audio:setSoundStage()
 end
 
 function UI:registerTextBox(box)
@@ -670,6 +674,23 @@ function UI:releaseMouse()
   self:setMouseReleased(true)
 end
 
+--! Dedicated hotkey function for toggling fullscreen
+function UI:fullscreenHotkey()
+  local toggle = self:toggleFullscreen()
+  if not toggle then
+    local err = {_S.errors.unavailable_screen_size}
+    self:addWindow(UIInformation(self, err))
+  end
+  -- Update the Options window, if open
+  local window = self:getWindow(UIOptions)
+  if window then
+    if toggle then window.fullscreen_button:toggle() end
+    window.fullscreen_panel:setLabel(self.app.fullscreen and _S.options_window.option_on or _S.options_window.option_off)
+  end
+end
+
+--! Turns fullscreen on and off
+--!return success true if toggle succeeded
 function UI:toggleFullscreen()
   local modes = self.app.modes
 
@@ -831,7 +852,7 @@ function UI:onKeyUp(rawchar)
   end
 end
 
-function UI:onEditingText(text, start, length)
+function UI:onEditingText(text, start, length) -- luacheck: ignore 212 keep args for child class
   -- Does nothing at the moment. We are handling text input ourselves.
 end
 
@@ -1029,19 +1050,29 @@ function UI:addWindow(window)
     end
     self.modal_windows[window.modal_class] = window
   end
+  if self.app.world and window:mustPause() then
+    self.app.world:setSpeed("Pause")
+    self.app.video:setBlueFilterActive(false) -- mustPause windows shouldn't cause tainting
+  end
   if window.modal_class == "main" or window.modal_class == "fullscreen" then
     self.editing_allowed = false -- do not allow editing rooms if main windows (build, furnish, hire) are open
   end
   Window.addWindow(self, window)
 end
 
-function UI:removeWindow(window)
-  if Window.removeWindow(self, window) then
-    local class = window.modal_class
-    if class and self.modal_windows[class] == window then
+function UI:removeWindow(closing_window)
+  if Window.removeWindow(self, closing_window) then
+    local class = closing_window.modal_class
+    if class and self.modal_windows[class] == closing_window then
       self.modal_windows[class] = nil
     end
-    if window.modal_class == "main" or window.modal_class == "fullscreen" then
+    if self.app.world and self.app.world:isCurrentSpeed("Pause") then
+      local pauseGame = self:checkForMustPauseWindows()
+      if not pauseGame and closing_window:mustPause() then
+        self.app.world:setSpeed(self.app.world.prev_speed)
+      end
+    end
+    if closing_window.modal_class == "main" or closing_window.modal_class == "fullscreen" then
       self.editing_allowed = true -- allow editing rooms again when main window is closed
     end
     return true
@@ -1050,6 +1081,15 @@ function UI:removeWindow(window)
   end
 end
 
+--! Function to check if we have any must pause windows open
+--!return (bool) Returns true if a must pause window is found
+function UI:checkForMustPauseWindows()
+  for _, window in pairs(self.windows) do
+    if window:mustPause() then return true end
+  end
+  return false
+end
+
 function UI:getCursorPosition(window)
   -- Given no argument, returns the cursor position in screen space
   -- Otherwise, returns the cursor position in the space of the given window
@@ -1080,6 +1120,10 @@ function UI:afterLoad(old, new)
     self.editing_allowed = true
   end
   self:setupGlobalKeyHandlers()
+
+  -- Cancel any saved screen movement from edge scrolling
+  self.tick_scroll_amount_mouse = nil
+
   Window.afterLoad(self, old, new)
 end
 
@@ -1112,6 +1156,11 @@ function UI:closeWindow()
     return false
   end
 
+  -- Stop the lose message being closed prematurely because we pressed "Escape" on the lose movie
+  if self.app.moviePlayer.playing then
+    return false
+  end
+
   -- Close the topmost window first
   local first = self.windows[1]
   if first.on_top and first.esc_closes then
