@@ -26,17 +26,35 @@ SOFTWARE.
 #ifdef CORSIX_TH_USE_FREETYPE2
 #include "th_gfx_font.h"
 #endif
+
 #include <algorithm>
 #include <cmath>
 #include <cstdio>
+#include <cstdlib>
 #include <cstring>
 #include <iostream>
 #include <limits>
+#include <memory>
 #include <new>
 #include <stdexcept>
 
 #include "th_map.h"
 
+#if SDL_VERSION_ATLEAST(2, 0, 10)
+
+//! How much to overdraw scaled sprites to ensure no gaps are visible.
+const float frect_overdraw = 0.002f;
+
+#define SDL_FRECT_UNIT float
+#else
+// On older SDL versions, floating point rendering was not available so we fall
+// back to integer methods / types.
+#define SDL_FRECT_UNIT int
+#define SDL_FRect SDL_Rect
+#define SDL_RenderCopyF SDL_RenderCopy
+#define SDL_RenderCopyExF SDL_RenderCopyEx
+#endif
+
 full_colour_renderer::full_colour_renderer(int iWidth, int iHeight)
     : width(iWidth), height(iHeight) {
   x = 0;
@@ -45,6 +63,14 @@ full_colour_renderer::full_colour_renderer(int iWidth, int iHeight)
 
 namespace {
 
+constexpr double pi = 3.14159265358979323846;
+
+//! The number of game ticks between jelly the jelly effect activating.
+constexpr int jelly_effect_period = 540;
+
+//! The number of game ticks the jelly effect is active for when it runs.
+constexpr int jelly_effect_duration = 90;
+
 //! Convert a colour to an equivalent grey scale level.
 /*!
     @param iOpacity Opacity of the pixel.
@@ -95,6 +121,30 @@ uint8_t convert_6bit_to_8bit_colour_component(uint8_t colour_component) {
       (colour_component & mask_6bit) * static_cast<double>(0xFF) / mask_6bit));
 }
 
+//! Get the intersection of two SDL_Rects. The output rect can be a
+//  pointer to either of the two input rects.
+/*!
+    @param a Pointer to first SDL_Rect to be intersected
+    @param b Pointer to second SDL_Rect to be intersected
+    @param[out] out The intersection of SDL_Rect |a| and |b|.
+ */
+void intersectRect(const SDL_Rect& a, const SDL_Rect& b, SDL_Rect* out) {
+  // The intersection of the rectangles is the higher of the lower bounds and
+  // the lower of the higher bounds, clamped to a zero size.
+  clip_rect::x_y_type maxX = std::min(a.x + a.w, b.x + b.w);
+  clip_rect::x_y_type maxY = std::min(a.y + a.h, b.y + b.h);
+  out->x = std::max(a.x, b.x);
+  out->y = std::max(a.y, b.y);
+  out->w = maxX - out->x;
+  out->h = maxY - out->y;
+
+  // Make sure that we clamp the values to 0.
+  if (out->w <= 0 || out->h <= 0) {
+    out->w = 0;
+    out->h = 0;
+  }
+}
+
 //! Get the enclosing rect of an SDL_Rect scaled by the given scale factor.
 //  Scaling the rect is location dependent, as non-integer scale factors
 //  require that same size rects be +/-1 pixel different in size depending
@@ -119,6 +169,45 @@ void getEnclosingScaleRect(const SDL_Rect* rect, double scale_factor,
   dst_rect->y = dst_y;
 }
 
+//! Get the scaled rect of an SDL_Rect scaled by the given scale factor.
+//  On SDL < 2.0.10 this falls back to using getEnclosingScaleRect.
+/*!
+    @param rect Pointer to the SDL_Rect to be scaled.
+    @param scale_factor Scale to be applied to the rectangle.
+    @param[out] dst_rect Enclosing SDL_FRect of rect scaled by scale_factor.
+ */
+void getScaleRect(const SDL_Rect* rect, double scale_factor,
+                  SDL_FRect* dst_rect) {
+#if SDL_VERSION_ATLEAST(2, 0, 10)
+  // If using SDL 2.0.10 or newer, we can use floats to get better precision
+  // on scaled rendering.
+  dst_rect->x = static_cast<float>(rect->x * scale_factor) - frect_overdraw;
+  dst_rect->y = static_cast<float>(rect->y * scale_factor) - frect_overdraw;
+  dst_rect->w = static_cast<float>(rect->w * scale_factor) + frect_overdraw;
+  dst_rect->h = static_cast<float>(rect->h * scale_factor) + frect_overdraw;
+#else
+  // Prior to SDL 2.0.10, fallback to using the enclosing integer SDL_Rect for
+  // scaled rendering.
+  getEnclosingScaleRect(rect, scale_factor, dst_rect);
+#endif
+}
+
+class scoped_color_mod {
+ public:
+  scoped_color_mod(SDL_Texture* pTexture, int r, int g, int b)
+      : texture(pTexture) {
+    int err = SDL_SetTextureColorMod(texture, r, g, b);
+    if (err < 0) {
+      throw std::runtime_error(SDL_GetError());
+    }
+  }
+
+  ~scoped_color_mod() { SDL_SetTextureColorMod(texture, 0xFF, 0xFF, 0xFF); }
+
+ private:
+  SDL_Texture* texture;
+};
+
 }  // namespace
 
 palette::palette() { colour_count = 0; }
@@ -200,6 +289,7 @@ void full_colour_renderer::decode_image(const uint8_t* pImg,
             iColour = makeGreyScale(iOpacity, pImg[0], pImg[1], pImg[2]);
           else
             iColour = palette::pack_argb(iOpacity, pImg[0], pImg[1], pImg[2]);
+
           push_pixel(iColour);
           pImg += 3;
           iLength--;
@@ -210,6 +300,7 @@ void full_colour_renderer::decode_image(const uint8_t* pImg,
       case 2:  // Fixed fully transparent pixels
       {
         static const uint32_t iTransparent = palette::pack_argb(0, 0, 0, 0);
+
         while (iLength > 0) {
           push_pixel(iTransparent);
           iLength--;
@@ -275,10 +366,74 @@ void wx_storing::store_argb(uint32_t pixel) {
   *alpha_data++ = palette::get_alpha(pixel);
 }
 
+class render_target::scoped_target_texture
+    : public render_target::scoped_buffer {
+ public:
+  scoped_target_texture(render_target* pTarget, int iX, int iY, int iWidth,
+                        int iHeight, bool bScale)
+      : target(pTarget),
+        previous_target(target->current_target),
+        rect({iX, iY, iWidth, iHeight}),
+        scale(bScale) {
+    if (!target->supports_target_textures) return;
+
+    texture = SDL_CreateTexture(target->renderer, SDL_PIXELFORMAT_ABGR8888,
+                                SDL_TEXTUREACCESS_TARGET, iWidth, iHeight);
+    if (SDL_SetRenderTarget(target->renderer, texture) != 0) {
+      SDL_DestroyTexture(texture);
+      texture = nullptr;
+      return;
+    }
+
+    // Clear the new texture to transparent/black.
+    SDL_RenderSetLogicalSize(target->renderer, rect.w, rect.h);
+    SDL_SetRenderDrawColor(target->renderer, 0, 0, 0, SDL_ALPHA_TRANSPARENT);
+    SDL_RenderClear(target->renderer);
+    target->current_target = this;
+  }
+
+  void offset(SDL_FRect& targetRect) {
+    targetRect.x -= static_cast<SDL_FRECT_UNIT>(rect.x);
+    targetRect.y -= static_cast<SDL_FRECT_UNIT>(rect.y);
+  }
+
+  double scale_factor() { return scale ? target->global_scale_factor : 1.0; }
+
+  bool is_target() { return texture; }
+
+  ~scoped_target_texture() override {
+    if (!texture) return;
+
+    // Restore previous context.
+    SDL_SetRenderTarget(target->renderer,
+                        previous_target ? previous_target->texture : nullptr);
+    SDL_RenderSetLogicalSize(
+        target->renderer,
+        previous_target ? previous_target->rect.w : target->width,
+        previous_target ? previous_target->rect.h : target->height);
+    SDL_SetTextureBlendMode(texture, SDL_BLENDMODE_BLEND);
+    target->current_target = previous_target;
+    if (scale) {
+      // If the target texture is already scaled, skip the global scale factor
+      // by drawing directly.
+      SDL_RenderCopy(target->renderer, texture, nullptr, &rect);
+    } else {
+      target->draw(texture, nullptr, &rect, 0);
+    }
+    SDL_DestroyTexture(texture);
+  }
+
+ private:
+  render_target* target;
+  scoped_target_texture* previous_target;
+  SDL_Rect rect;
+  bool scale;
+  SDL_Texture* texture = nullptr;
+};
+
 render_target::render_target()
     : window(nullptr),
       renderer(nullptr),
-      zoom_texture(nullptr),
       pixel_format(nullptr),
       blue_filter_active(false),
       game_cursor(nullptr),
@@ -296,15 +451,7 @@ bool render_target::create(const render_target_creation_params* pParams) {
 
   direct_zoom = pParams->direct_zoom;
 
-  if (direct_zoom) {
-    // When scaling rendering, linear introduces semi-transparent gaps at
-    // transparent edges within tiles. Using nearest ensures that we can scale
-    // angled transparent tile edges without seams.
-    SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "nearest");
-  } else {
-    SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "linear");
-  }
-
+  SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "linear");
   pixel_format = SDL_AllocFormat(SDL_PIXELFORMAT_ABGR8888);
   window =
       SDL_CreateWindow("CorsixTH", SDL_WINDOWPOS_UNDEFINED,
@@ -360,16 +507,13 @@ bool render_target::update(const render_target_creation_params* pParams) {
 }
 
 void render_target::destroy() {
+  zoom_buffer.reset();
+
   if (pixel_format) {
     SDL_FreeFormat(pixel_format);
     pixel_format = nullptr;
   }
 
-  if (zoom_texture) {
-    SDL_DestroyTexture(zoom_texture);
-    zoom_texture = nullptr;
-  }
-
   if (renderer) {
     SDL_DestroyRenderer(renderer);
     renderer = nullptr;
@@ -382,13 +526,21 @@ void render_target::destroy() {
 }
 
 bool render_target::set_scale_factor(double fScale, scaled_items eWhatToScale) {
-  flush_zoom_buffer();
+  zoom_buffer.reset();
   scale_bitmaps = false;
 
   if (fScale <= 0.000) {
     return false;
   } else if (eWhatToScale == scaled_items::all && direct_zoom) {
     global_scale_factor = fScale;
+    if ((SDL_GetWindowFlags(window) & SDL_WINDOW_FULLSCREEN_DESKTOP) ==
+        SDL_WINDOW_FULLSCREEN_DESKTOP) {
+      // Drawing to an intermediate screen sized buffer when fullscreen results
+      // in noticeably better text rendering quality.
+      zoom_buffer =
+          std::make_unique<scoped_target_texture>(this, 0, 0, width, height,
+                                                  /* bScale = */ true);
+    }
     return true;
   } else if (eWhatToScale == scaled_items::all && supports_target_textures) {
     // Draw everything from now until the next scale to zoom_texture
@@ -397,25 +549,19 @@ bool render_target::set_scale_factor(double fScale, scaled_items eWhatToScale) {
     int virtWidth = static_cast<int>(width / fScale);
     int virtHeight = static_cast<int>(height / fScale);
 
-    zoom_texture =
-        SDL_CreateTexture(renderer, SDL_PIXELFORMAT_ABGR8888,
-                          SDL_TEXTUREACCESS_TARGET, virtWidth, virtHeight);
-
-    SDL_RenderSetLogicalSize(renderer, virtWidth, virtHeight);
-    if (SDL_SetRenderTarget(renderer, zoom_texture) != 0) {
+    zoom_buffer = std::make_unique<scoped_target_texture>(
+        this, 0, 0, virtWidth, virtHeight, /* bScale = */ false);
+    if (!zoom_buffer->is_target()) {
+      global_scale_factor = 1.0;
       std::cout << "Warning: Could not render to zoom texture - "
                 << SDL_GetError() << std::endl;
 
-      SDL_RenderSetLogicalSize(renderer, width, height);
-      SDL_DestroyTexture(zoom_texture);
-      zoom_texture = nullptr;
       return false;
     }
 
-    // Clear the new texture to transparent/black.
-    SDL_SetRenderDrawColor(renderer, 0, 0, 0, SDL_ALPHA_TRANSPARENT);
-    SDL_RenderClear(renderer);
-
+    // When zoom_buffer is allocated with bScale = false, this is only used for
+    // the scale to commit the zoom buffer back to the screen at.
+    global_scale_factor = fScale;
     return true;
   } else if (0.999 <= fScale && fScale <= 1.001) {
     return true;
@@ -446,7 +592,7 @@ bool render_target::start_frame() {
 }
 
 bool render_target::end_frame() {
-  flush_zoom_buffer();
+  zoom_buffer.reset();
 
   // End the frame by adding the cursor and possibly a filter.
   if (game_cursor) {
@@ -482,7 +628,7 @@ void render_target::set_window_grab(bool bActivate) {
 bool render_target::fill_rect(uint32_t iColour, int iX, int iY, int iW,
                               int iH) {
   SDL_Rect rcDest = {iX, iY, iW, iH};
-  getEnclosingScaleRect(&rcDest, global_scale_factor, &rcDest);
+  getEnclosingScaleRect(&rcDest, draw_scale(), &rcDest);
 
   Uint8 r, g, b, a;
   SDL_GetRGBA(iColour, pixel_format, &r, &g, &b, &a);
@@ -494,49 +640,50 @@ bool render_target::fill_rect(uint32_t iColour, int iX, int iY, int iW,
   return true;
 }
 
-void render_target::get_clip_rect(clip_rect* pRect) const {
-  SDL_RenderGetClipRect(renderer, reinterpret_cast<SDL_Rect*>(pRect));
-  // SDL returns empty rect when clipping is disabled -> return full rect for
-  // CTH
-  if (SDL_RectEmpty(pRect)) {
-    pRect->x = pRect->y = 0;
-    pRect->w = width;
-    pRect->h = height;
-  }
-
-  if (apply_opengl_clip_fix) {
-    int renderWidth, renderHeight;
-    SDL_RenderGetLogicalSize(renderer, &renderWidth, &renderHeight);
-    pRect->y = renderHeight - pRect->y - pRect->h;
-  }
-
-  getEnclosingScaleRect(pRect, 1.0 / global_scale_factor, pRect);
+render_target::scoped_clip::scoped_clip(render_target* pTarget,
+                                        const clip_rect* pRect)
+    : target(pTarget) {
+  target->push_clip_rect(pRect);
 }
 
-void render_target::set_clip_rect(const clip_rect* pRect) {
-  // Full clip rect for CTH means clipping disabled
-  if (pRect == nullptr || (pRect->w == width && pRect->h == height)) {
-    SDL_RenderSetClipRect(renderer, nullptr);
-    return;
-  }
+render_target::scoped_clip::~scoped_clip() { target->pop_clip_rect(); }
 
-  SDL_Rect SDLRect;
-  getEnclosingScaleRect(pRect, global_scale_factor, &SDLRect);
+void render_target::push_clip_rect(const clip_rect* pRect) {
+  const SDL_Rect* previous_clip = nullptr;
+  if (!clip_rects.empty()) {
+    previous_clip = &clip_rects.top();
+  }
+  clip_rects.push(SDL_Rect());
+  SDL_Rect& clip = clip_rects.top();
+  getEnclosingScaleRect(pRect, draw_scale(), &clip);
+  if (previous_clip) {
+    // Ensure that this new clip does not escape any previous clips applied.
+    intersectRect(clip, *previous_clip, &clip);
+  }
 
   // For some reason, SDL treats an empty rect (h or w <= 0) as if you turned
   // off clipping, so we replace it with a rect that's outside our viewport.
   const SDL_Rect rcBogus = {-2, -2, 1, 1};
-  if (SDL_RectEmpty(&SDLRect)) {
-    SDLRect = rcBogus;
+  if (SDL_RectEmpty(&clip)) {
+    clip = rcBogus;
   }
 
   if (apply_opengl_clip_fix) {
     int renderWidth, renderHeight;
     SDL_RenderGetLogicalSize(renderer, &renderWidth, &renderHeight);
-    SDLRect.y = renderHeight - SDLRect.y - SDLRect.h;
+    clip.y = renderHeight - clip.y - clip.h;
   }
 
-  SDL_RenderSetClipRect(renderer, &SDLRect);
+  SDL_RenderSetClipRect(renderer, &clip);
+}
+
+void render_target::pop_clip_rect() {
+  clip_rects.pop();
+  if (clip_rects.empty()) {
+    SDL_RenderSetClipRect(renderer, nullptr);
+  } else {
+    SDL_RenderSetClipRect(renderer, &clip_rects.top());
+  }
 }
 
 int render_target::get_width() const {
@@ -598,19 +745,6 @@ bool render_target::should_scale_bitmaps(double* pFactor) {
   return true;
 }
 
-void render_target::flush_zoom_buffer() {
-  if (zoom_texture == nullptr) {
-    return;
-  }
-
-  SDL_SetRenderTarget(renderer, nullptr);
-  SDL_RenderSetLogicalSize(renderer, width, height);
-  SDL_SetTextureBlendMode(zoom_texture, SDL_BLENDMODE_BLEND);
-  SDL_RenderCopy(renderer, zoom_texture, nullptr, nullptr);
-  SDL_DestroyTexture(zoom_texture);
-  zoom_texture = nullptr;
-}
-
 namespace {
 
 //! Convert legacy 8bpp sprite data to recoloured 32bpp data, using special
@@ -658,7 +792,11 @@ SDL_Texture* render_target::create_palettized_texture(
   full_colour_storing oRenderer(pARGBPixels, iWidth, iHeight);
   oRenderer.decode_image(pPixels, pPalette, iSpriteFlags);
 
+  if (iSpriteFlags & thdf_nearest)
+    SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "nearest");
   SDL_Texture* pTexture = create_texture(iWidth, iHeight, pARGBPixels);
+  if (iSpriteFlags & thdf_nearest)
+    SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "linear");
   delete[] pARGBPixels;
   return pTexture;
 }
@@ -711,40 +849,52 @@ void render_target::draw(SDL_Texture* pTexture, const SDL_Rect* prcSrcRect,
   if (iFlags & thdf_flip_horizontal) iSDLFlip |= SDL_FLIP_HORIZONTAL;
   if (iFlags & thdf_flip_vertical) iSDLFlip |= SDL_FLIP_VERTICAL;
 
-  SDL_Rect scaledDstRect;
-  getEnclosingScaleRect(prcDstRect, global_scale_factor, &scaledDstRect);
+  SDL_FRect scaledDstRect;
+  getScaleRect(prcDstRect, draw_scale(), &scaledDstRect);
+  if (current_target) current_target->offset(scaledDstRect);
   if (iSDLFlip != 0) {
-    SDL_RenderCopyEx(renderer, pTexture, prcSrcRect, &scaledDstRect, 0, nullptr,
-                     (SDL_RendererFlip)iSDLFlip);
+    SDL_RenderCopyExF(renderer, pTexture, prcSrcRect, &scaledDstRect, 0,
+                      nullptr, (SDL_RendererFlip)iSDLFlip);
   } else {
-    SDL_RenderCopy(renderer, pTexture, prcSrcRect, &scaledDstRect);
+    SDL_RenderCopyF(renderer, pTexture, prcSrcRect, &scaledDstRect);
   }
 }
 
-void render_target::draw_line(line* pLine, int iX, int iY) {
+void render_target::draw_line(line_sequence* pLine, int iX, int iY) {
   SDL_SetRenderDrawColor(renderer, pLine->red, pLine->green, pLine->blue,
                          pLine->alpha);
 
-  double lastX, lastY;
-  lastX = pLine->first_operation->x;
-  lastY = pLine->first_operation->y;
-
-  line::line_operation* op =
-      (line::line_operation*)(pLine->first_operation->next);
-  while (op) {
-    if (op->type == line::line_operation_type::line) {
-      SDL_RenderDrawLine(
-          renderer, static_cast<int>(lastX + iX), static_cast<int>(lastY + iY),
-          static_cast<int>(op->x + iX), static_cast<int>(op->y + iY));
+  double scale = draw_scale();
+  double lastX = pLine->line_elements[0].x;
+  double lastY = pLine->line_elements[0].y;
+  for (const line_sequence::line_element& op : pLine->line_elements) {
+    if (op.type == line_sequence::line_command::line) {
+      SDL_RenderDrawLine(renderer, static_cast<int>((lastX + iX) * scale),
+                         static_cast<int>((lastY + iY) * scale),
+                         static_cast<int>((op.x + iX) * scale),
+                         static_cast<int>((op.y + iY) * scale));
     }
 
-    lastX = op->x;
-    lastY = op->y;
-
-    op = (line::line_operation*)(op->next);
+    lastX = op.x;
+    lastY = op.y;
   }
 }
 
+std::unique_ptr<render_target::scoped_buffer>
+render_target::begin_intermediate_drawing(int iX, int iY, int iWidth,
+                                          int iHeight) {
+  // We only need an intermediate drawing if there is active scaling.
+  if (draw_scale() == 1.0) return nullptr;
+
+  return std::make_unique<scoped_target_texture>(this, iX, iY, iWidth, iHeight,
+                                                 /* bScale = */ false);
+}
+
+double render_target::draw_scale() const {
+  if (current_target) return current_target->scale_factor();
+  return global_scale_factor;
+}
+
 raw_bitmap::raw_bitmap() {
   texture = nullptr;
   bitmap_palette = nullptr;
@@ -1064,7 +1214,8 @@ bool sprite_sheet::get_sprite_average_colour(size_t iSprite,
 }
 
 void sprite_sheet::draw_sprite(render_target* pCanvas, size_t iSprite, int iX,
-                               int iY, uint32_t iFlags) {
+                               int iY, uint32_t iFlags, size_t effect_ticks,
+                               animation_effect effect) {
   if (iSprite >= sprite_count || pCanvas == nullptr || pCanvas != target)
     return;
   sprite& sprite = sprites[iSprite];
@@ -1074,8 +1225,8 @@ void sprite_sheet::draw_sprite(render_target* pCanvas, size_t iSprite, int iX,
   if (!pTexture) {
     if (sprite.data == nullptr) return;
 
-    uint32_t iSprFlags =
-        (sprite.sprite_flags & ~thdf_alt32_mask) | thdf_alt32_plain;
+    uint32_t iSprFlags = (sprite.sprite_flags & ~thdf_alt32_mask) |
+                         thdf_alt32_plain | (iFlags & thdf_nearest);
     pTexture = target->create_palettized_texture(
         sprite.width, sprite.height, sprite.data, palette, iSprFlags);
     sprite.texture = pTexture;
@@ -1088,10 +1239,58 @@ void sprite_sheet::draw_sprite(render_target* pCanvas, size_t iSprite, int iX,
     }
   }
 
-  SDL_Rect rcSrc = {0, 0, sprite.width, sprite.height};
-  SDL_Rect rcDest = {iX, iY, sprite.width, sprite.height};
+  std::unique_ptr<scoped_color_mod> color_mod;
+  if (effect == animation_effect::glowing) {
+    // We want this to vary between 155 -> 205 -> 255.
+    // Use the target cycle length (15 cycles) to calculate the angle from 0 to
+    // 2*pi.
+    int currentVariation = static_cast<int>(
+        sin(static_cast<double>(effect_ticks % 15) / 15.0 * 2 * pi) * 50);
+    color_mod = std::make_unique<scoped_color_mod>(pTexture, 0,
+                                                   205 + currentVariation, 0);
+  }
 
-  pCanvas->draw(pTexture, &rcSrc, &rcDest, iFlags);
+  if (effect == animation_effect::jelly &&
+      effect_ticks % jelly_effect_period < jelly_effect_duration) {
+    int jelly_tick = static_cast<int>(effect_ticks % jelly_effect_period);
+    // Draw the sprite a few lines at a time following a sine wave x offset.
+    // To cut down on the number of draw calls, we will draw all of the lines
+    // that have the same x offset in a single draw call. This results in about
+    // a third of the draw calls as drawing each line individually.
+    // TODO: We could move this effect into render_target which is aware of the
+    // screen scale and could optimize this further when zooming out.
+    int y1 = 0;
+    int x_offset = 0;
+    // Scale the effect up as it ramps in and down as it finishes.
+    int scale =
+        std::min(jelly_tick, std::min(jelly_effect_duration - jelly_tick, 2));
+    for (int y2 = 0; y2 <= sprite.height; y2++) {
+      // TODO: Ideally this should use the offset of the current line from
+      // the map location so that multiple layers have the same jelly offset
+      // at the same vertical line. We can't just use iY because it varies as
+      // the user scrolls or zooms or the character walks.
+      int offset = static_cast<int>(
+          sin((y2 / 16.0 + static_cast<double>(effect_ticks % 50) / 50) * 2 *
+              pi) *
+          scale);
+      if (x_offset != offset || y2 == sprite.height) {
+        if (y2 > y1) {
+          // If the current offset rounds to a different value, render the
+          // previous offset and start a new offset.
+          SDL_Rect rcSrc = {0, y1, sprite.width, y2 - y1};
+          SDL_Rect rcDest = {iX + x_offset, iY + y1, sprite.width, y2 - y1};
+          pCanvas->draw(pTexture, &rcSrc, &rcDest, iFlags);
+        }
+        y1 = y2;
+        x_offset = offset;
+      }
+    }
+  } else {
+    SDL_Rect rcSrc = {0, 0, sprite.width, sprite.height};
+    SDL_Rect rcDest = {iX, iY, sprite.width, sprite.height};
+
+    pCanvas->draw(pTexture, &rcSrc, &rcDest, iFlags);
+  }
 }
 
 void sprite_sheet::wx_draw_sprite(size_t iSprite, uint8_t* pRGBData,
@@ -1310,80 +1509,59 @@ void cursor::draw(render_target* pCanvas, int iX, int iY) {
 #endif
 }
 
-line::line() { initialize(); }
+line_sequence::line_sequence() { initialize(); }
 
-line::~line() {
-  line_operation* op = first_operation;
-  while (op) {
-    line_operation* next = (line_operation*)(op->next);
-    delete (op);
-    op = next;
-  }
-}
-
-void line::initialize() {
+void line_sequence::initialize() {
   width = 1;
   red = 0;
   green = 0;
   blue = 0;
   alpha = 255;
+  line_elements.clear();
 
   // We start at 0,0
-  first_operation = new line_operation(line_operation_type::move, 0, 0);
-  current_operation = first_operation;
+  move_to(0.0, 0.0);
 }
 
-void line::move_to(double fX, double fY) {
-  line_operation* previous = current_operation;
-  current_operation = new line_operation(line_operation_type::move, fX, fY);
-  previous->next = current_operation;
+void line_sequence::move_to(double fX, double fY) {
+  line_elements.emplace_back(line_command::move, fX, fY);
 }
 
-void line::line_to(double fX, double fY) {
-  line_operation* previous = current_operation;
-  current_operation = new line_operation(line_operation_type::line, fX, fY);
-  previous->next = current_operation;
+void line_sequence::line_to(double fX, double fY) {
+  line_elements.emplace_back(line_command::line, fX, fY);
 }
 
-void line::set_width(double pLineWidth) { width = pLineWidth; }
+void line_sequence::set_width(double pLineWidth) { width = pLineWidth; }
 
-void line::set_colour(uint8_t iR, uint8_t iG, uint8_t iB, uint8_t iA) {
+void line_sequence::set_colour(uint8_t iR, uint8_t iG, uint8_t iB, uint8_t iA) {
   red = iR;
   green = iG;
   blue = iB;
   alpha = iA;
 }
 
-void line::draw(render_target* pCanvas, int iX, int iY) {
+void line_sequence::draw(render_target* pCanvas, int iX, int iY) {
   pCanvas->draw_line(this, iX, iY);
 }
 
-void line::persist(lua_persist_writer* pWriter) const {
+void line_sequence::persist(lua_persist_writer* pWriter) const {
   pWriter->write_uint(static_cast<uint32_t>(red));
   pWriter->write_uint(static_cast<uint32_t>(green));
   pWriter->write_uint(static_cast<uint32_t>(blue));
   pWriter->write_uint(static_cast<uint32_t>(alpha));
-  pWriter->write_float(width);
-
-  line_operation* op = (line_operation*)(first_operation->next);
-  uint32_t numOps = 0;
-  for (; op; numOps++) {
-    op = (line_operation*)(op->next);
-  }
+  pWriter->write_float<double>(width);
 
+  uint32_t numOps = static_cast<uint32_t>(line_elements.size());
   pWriter->write_uint(numOps);
 
-  op = (line_operation*)(first_operation->next);
-  while (op) {
-    pWriter->write_uint(static_cast<uint32_t>(op->type));
-    pWriter->write_float<double>(op->x);
-    pWriter->write_float(op->y);
-
-    op = (line_operation*)(op->next);
+  for (const line_element& op : line_elements) {
+    pWriter->write_uint(static_cast<uint32_t>(op.type));
+    pWriter->write_float<double>(op.x);
+    pWriter->write_float<double>(op.y);
   }
 }
 
-void line::depersist(lua_persist_reader* pReader) {
+void line_sequence::depersist(lua_persist_reader* pReader) {
   initialize();
 
   pReader->read_uint(red);
@@ -1408,9 +1586,9 @@ void line::depersist(lua_persist_reader* pReader) {
       return;
     }
 
-    if (type_val == static_cast<uint32_t>(line_operation_type::move)) {
+    if (type_val == static_cast<uint32_t>(line_command::move)) {
       move_to(fX, fY);
-    } else if (type_val == static_cast<uint32_t>(line_operation_type::line)) {
+    } else if (type_val == static_cast<uint32_t>(line_command::line)) {
       line_to(fX, fY);
     }
   }
