@@ -935,41 +935,23 @@ void level_map::set_all_wall_draw_flags(uint8_t iFlags) {
   }
 }
 
-// Definition is in th_gfx.h so this should move
-void clip_rect_intersection(clip_rect& rcClip, const clip_rect& rcIntersect) {
-  // The intersection of the rectangles is the higher of the lower bounds and
-  // the lower of the higher bounds, clamped to a zero size.
-  clip_rect::x_y_type maxX = static_cast<uint16_t>(
-      std::min(rcClip.x + rcClip.w, rcIntersect.x + rcIntersect.w));
-  clip_rect::x_y_type maxY = static_cast<uint16_t>(
-      std::min(rcClip.y + rcClip.h, rcIntersect.y + rcIntersect.h));
-  rcClip.x = std::max(rcClip.x, rcIntersect.x);
-  rcClip.y = std::max(rcClip.y, rcIntersect.y);
-  rcClip.w = maxX - rcClip.x;
-  rcClip.h = maxY - rcClip.y;
-
-  // Make sure that we clamp the values to 0.
-  if (rcClip.w <= 0 || rcClip.h <= 0) {
-    rcClip.w = 0;
-    rcClip.h = 0;
-  }
-}
-
 void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
                      int iWidth, int iHeight, int iCanvasX,
                      int iCanvasY) const {
   /*
      The map is drawn in two passes, with each pass done one scanline at a
      time (a scanline is a list of tiles with the same screen Y coordinate).
-     The first pass does floor tiles, as the entire floor needs to be painted
-     below anything else (for example, see the walking north through a door
-     animation, which needs to paint over the floor of the scanline below the
-     animation). On the second pass, walls and entities are drawn, with the
-     order controlled such that entities appear in the right order relative to
-     the walls around them. For each scanline, the following is done:
+     The first pass draws the floor tiles and floor shadows, as the entire
+     floor needs to be painted below anything else (for example, see the
+     walking north through a door animation or death animation, which needs
+     to paint over the floor of the scanline below the animation). On the
+     second pass, walls and entities are drawn, with the order controlled
+     such that entities appear in the right order relative to the walls
+     around them. For each scanline, the following is done:
 
      1st pass:
       1) For each tile, left to right, the floor tile (layer 0)
+      2) The floor shadow of that tile.
      2nd pass:
       1) For each tile, right to left, the north wall, then the early entities
       2) For each tile, left to right, the west wall, then the late entities
@@ -984,43 +966,45 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
   rcClip.y = static_cast<clip_rect::x_y_type>(iCanvasY);
   rcClip.w = static_cast<clip_rect::w_h_type>(iWidth);
   rcClip.h = static_cast<clip_rect::w_h_type>(iHeight);
-  pCanvas->set_clip_rect(&rcClip);
+  render_target::scoped_clip clip(pCanvas, &rcClip);
 
-  // 1st pass
-  pCanvas->start_nonoverlapping_draws();
   for (map_tile_iterator itrNode1(this, iScreenX, iScreenY, iWidth, iHeight);
        itrNode1; ++itrNode1) {
+    // First, draw the floor tile as it should be below everything else.
     int iH = 32;
     unsigned int iBlock = itrNode1->iBlock[0];
     blocks->get_sprite_size(iBlock & 0xFF, nullptr, &iH);
     blocks->draw_sprite(
         pCanvas, iBlock & 0xFF,
         itrNode1.tile_x_position_on_screen() + iCanvasX - 32,
-        itrNode1.tile_y_position_on_screen() + iCanvasY - iH + 32, iBlock >> 8);
+        itrNode1.tile_y_position_on_screen() + iCanvasY - iH + 32,
+        (iBlock >> 8) | thdf_nearest);
+
+    // Draw floor shadows immediately after floor tiles ensuring that all
+    // shadow pixels are drawn onto freshly drawn opaque floor tile pixels.
+    if (itrNode1->flags.shadow_full) {
+      blocks->draw_sprite(pCanvas, 74,
+                          itrNode1.tile_x_position_on_screen() + iCanvasX - 32,
+                          itrNode1.tile_y_position_on_screen() + iCanvasY,
+                          thdf_alpha_75 | thdf_nearest);
+    } else if (itrNode1->flags.shadow_half) {
+      blocks->draw_sprite(pCanvas, 75,
+                          itrNode1.tile_x_position_on_screen() + iCanvasX - 32,
+                          itrNode1.tile_y_position_on_screen() + iCanvasY,
+                          thdf_alpha_75 | thdf_nearest);
+    }
   }
-  pCanvas->finish_nonoverlapping_draws();
 
-  // 2nd pass
   bool bFirst = true;
   map_scanline_iterator formerIterator;
-  for (map_tile_iterator itrNode2(this, iScreenX, iScreenY, iWidth, iHeight);
-       itrNode2; ++itrNode2) {
-    if (itrNode2->flags.shadow_full) {
-      blocks->draw_sprite(
-          pCanvas, 74, itrNode2.tile_x_position_on_screen() + iCanvasX - 32,
-          itrNode2.tile_y_position_on_screen() + iCanvasY, thdf_alpha_75);
-    } else if (itrNode2->flags.shadow_half) {
-      blocks->draw_sprite(
-          pCanvas, 75, itrNode2.tile_x_position_on_screen() + iCanvasX - 32,
-          itrNode2.tile_y_position_on_screen() + iCanvasY, thdf_alpha_75);
-    }
-
-    if (!itrNode2.is_last_on_scanline()) {
+  for (map_tile_iterator itrNode1(this, iScreenX, iScreenY, iWidth, iHeight);
+       itrNode1; ++itrNode1) {
+    if (!itrNode1.is_last_on_scanline()) {
       continue;
     }
 
     for (map_scanline_iterator itrNode(
-             itrNode2, map_scanline_iterator_direction::backward, iCanvasX,
+             itrNode1, map_scanline_iterator_direction::backward, iCanvasX,
              iCanvasY);
          itrNode; ++itrNode) {
       int iH;
@@ -1028,20 +1012,18 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
       if (iBlock != 0 && blocks->get_sprite_size(iBlock & 0xFF, nullptr, &iH) &&
           iH > 0) {
         blocks->draw_sprite(pCanvas, iBlock & 0xFF, itrNode.x() - 32,
-                            itrNode.y() - iH + 32, iBlock >> 8);
+                            itrNode.y() - iH + 32,
+                            (iBlock >> 8) | thdf_nearest);
         if (itrNode->flags.shadow_wall) {
-          clip_rect rcOldClip, rcNewClip;
-          pCanvas->get_clip_rect(&rcOldClip);
+          clip_rect rcNewClip;
           rcNewClip.x = static_cast<clip_rect::x_y_type>(itrNode.x() - 32);
           rcNewClip.y =
               static_cast<clip_rect::x_y_type>(itrNode.y() - iH + 32 + 4);
           rcNewClip.w = static_cast<clip_rect::w_h_type>(64);
           rcNewClip.h = static_cast<clip_rect::w_h_type>(86 - 4);
-          clip_rect_intersection(rcNewClip, rcOldClip);
-          pCanvas->set_clip_rect(&rcNewClip);
+          render_target::scoped_clip clip(pCanvas, &rcNewClip);
           blocks->draw_sprite(pCanvas, 156, itrNode.x() - 32, itrNode.y() - 56,
-                              thdf_alpha_75);
-          pCanvas->set_clip_rect(&rcOldClip);
+                              thdf_alpha_75 | thdf_nearest);
         }
       }
       drawable* pItem = static_cast<drawable*>(itrNode->oEarlyEntities.next);
@@ -1052,7 +1034,7 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
     }
 
     map_scanline_iterator itrNode(
-        itrNode2, map_scanline_iterator_direction::forward, iCanvasX, iCanvasY);
+        itrNode1, map_scanline_iterator_direction::forward, iCanvasX, iCanvasY);
     if (!bFirst) {
       // since the scanline count from one THMapScanlineIterator to
       // another can differ synchronization between the current iterator
@@ -1072,13 +1054,15 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
       if (iBlock != 0 && blocks->get_sprite_size(iBlock & 0xFF, nullptr, &iH) &&
           iH > 0) {
         blocks->draw_sprite(pCanvas, iBlock & 0xFF, itrNode.x() - 32,
-                            itrNode.y() - iH + 32, iBlock >> 8);
+                            itrNode.y() - iH + 32,
+                            (iBlock >> 8) | thdf_nearest);
       }
       iBlock = itrNode->iBlock[3];
       if (iBlock != 0 && blocks->get_sprite_size(iBlock & 0xFF, nullptr, &iH) &&
           iH > 0) {
         blocks->draw_sprite(pCanvas, iBlock & 0xFF, itrNode.x() - 32,
-                            itrNode.y() - iH + 32, iBlock >> 8);
+                            itrNode.y() - iH + 32,
+                            (iBlock >> 8) | thdf_nearest);
       }
       iBlock = itrNode->iBlock[1];
       if (iBlock != 0 && blocks->get_sprite_size(iBlock & 0xFF, nullptr, &iH) &&
@@ -1146,20 +1130,19 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
           if (iBlock != 0 &&
               blocks->get_sprite_size(iBlock & 0xFF, nullptr, &iH) && iH > 0) {
             blocks->draw_sprite(pCanvas, iBlock & 0xFF, itrNode.x() - 96,
-                                itrNode.y() - iH + 32, iBlock >> 8);
+                                itrNode.y() - iH + 32,
+                                (iBlock >> 8) | thdf_nearest);
             if (itrNode.get_previous_tile()->flags.shadow_wall) {
-              clip_rect rcOldClip, rcNewClip;
-              pCanvas->get_clip_rect(&rcOldClip);
+              clip_rect rcNewClip;
               rcNewClip.x = static_cast<clip_rect::x_y_type>(itrNode.x() - 96);
               rcNewClip.y =
                   static_cast<clip_rect::x_y_type>(itrNode.y() - iH + 32 + 4);
               rcNewClip.w = static_cast<clip_rect::w_h_type>(64);
               rcNewClip.h = static_cast<clip_rect::w_h_type>(86 - 4);
-              clip_rect_intersection(rcNewClip, rcOldClip);
-              pCanvas->set_clip_rect(&rcNewClip);
+              render_target::scoped_clip clip(pCanvas, &rcNewClip);
               blocks->draw_sprite(pCanvas, 156, itrNode.x() - 96,
-                                  itrNode.y() - 56, thdf_alpha_75);
-              pCanvas->set_clip_rect(&rcOldClip);
+                                  itrNode.y() - 56,
+                                  thdf_alpha_75 | thdf_nearest);
             }
           }
 
@@ -1195,8 +1178,6 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
                          itrNode.tile_x(), itrNode.tile_y());
     }
   }
-
-  pCanvas->set_clip_rect(nullptr);
 }
 
 drawable* level_map::hit_test(int iTestX, int iTestY) const {
